;********************************************************************
; Monitor (gelb) berichtigte Orginalfassung von 1985
;
; Video-Monitor V7.5
; (C)hms85 Reinhard Gler  13.11.85
; 
; Editor:		notepad++ auf TAB=6
; Assembler:	TASM (C)Thomas N.Anderson
;                             
; Neu aufgebaut W.Rmer 2014-2018
;
;	25.01.2018 - Quellcode ueberarbeitet
;			 TAB auf 6 gesetzt, damit Listfile = Quellcode ist
;			 Anpassungen fuer TASM	(.org,.equ,.db etc.)
;
;	2014	     - alle Quellprogrammteile zusammengefasst
;			 Sprungtabellen auf Symbolische Sprnge gendert.
;			 Damit knnen nderungen an Tabellen etc einfach
;			 durchgefhrt werden 
;			 mit "8085 Simulator IDE" assembliert
; 
;--------------------------------------------------------------------
;
; Constante
;
stpff		.equ	1ah		; STP-Flipflop
vidreg	.equ	28h		; CRT-Reg-Adr.
vidat		.equ	29h		; CRT-Dat-Adr.
vidctl	.equ	2ah		; Format/Blanking Update
fdctrl	.equ	40h		; FDC/Adresslatch
belout	.equ	0b9h		; Schnarre
;
rst5on	.equ	1eh		; 8085
rstcon	.equ	02h		; NSC800
rst6on	.equ	1dh
rstbon	.equ	04h
rst7on	.equ	1bh
rstaon	.equ	08h
rstoff	.equ	17h
nscoff	.equ	00h
;
; Adressen vom Monitor #0
;
timprl	.equ	7h
rst1		.equ	8h
dely1		.equ	0bh
rst2		.equ	10h
lo		.equ	4fh
csinit	.equ	55h
loinit	.equ	58h
maxram	.equ	5bh
time		.equ	61h
v24ini	.equ	64h 		; Adresse, kein JMP!
writur	.equ	79h
readur	.equ	7ch
save		.equ	7fh
restor	.equ	82h
fill		.equ	85h
copy		.equ	88h
casot		.equ	8bh
casin		.equ	8eh
exchan	.equ	91h
jback		.equ	0cdh		; Jump Back PROMO
;
;---------------------------------------------------------
;
; bis  2DFFh     :Anwender-RAM-Bereich
; ab 2E00h Buffer-adressen
;
;
rambeg:	.equ	2800h		; CPU-RAM
vidram:	.equ	3000h		; Anfang Video-RAM
vidend:	.equ	3780h		; hinter Bildausschnitt
promsw:	.equ	7a00h		; Ende Editor-PROM
;	
buff_th:	.equ	2e00h     	; Buffer f.Thermo-Drucker
buff_fkt:	.equ	2e20h		; Fct-Tasten
buff_vid:	.equ	2e4eh 	; Video
buff_ln:	.equ	2e60h     	; Linebuffer 80 Zeichen
buff_mon:	.equ	2eb0h		; Monitor
buff_sta:	.equ	2f00h		; Stack
buff_ass:	.equ	2f00h		; Assembler
buff_usr:	.equ	2f64h		; Userstack
buff_fdc:	.equ	2f70h		; FDC-Routinen
buff_irq:	.equ	2f8ch		; Interuptverteiler
buff_uhr:	.equ	2fdch		; Uhr-BCD
buff_asc:	.equ	2feah 	; Uhr-ASCII
;
;----------------------------------------------
;
	.org    1000h          ; CPU-ROM Platz #1
;
kalt:		jmp	kalt_ 	; Kaltstart
warm:		jmp	warm_ 	; Warmstart
co:		jmp	co_   	; Console Output
ci:		jmp	ci_   	; Console Input
csts:		jmp	csts_ 	; Console Status
break:	jmp	break_	; Programmabbruch?
parin:	jmp	parin_	; Parameter einlesen
hxtest:	jmp	hxtes_	; ASCII/HEX-Umsetzung
compar:	jmp	compa_	; MEM-Differenz ermitteln
sample:	jmp	sampl_	; Zeichenvorrat ausgeben
v24ot: 	jmp	v24ot_	; PO ber V.24
string:	jmp	strin_	; Video-Ausgabe
v24in: 	jmp	v24in_	; RI ber V.24
error: 	jmp	error_	; Fehlermeldung
undrln:	jmp	undrl_	; String invers
clrvid:	jmp	clrvi_	; Clear Video-RAM
clrlin:	jmp	clrli_	; Bildzeile lschen
crspos:	jmp	crspo_	; Cursor positionieren
crson: 	jmp	crson_	; Cursor einschalten
crsoff:	jmp	crsof_	; Cursor ausschalten
adrot: 	jmp	adrot_	; (H&L) ausgeben
bytot: 	jmp	bytot_	; (ACC) ausgeben
bytbin:	jmp	bytbi_	; (ACC) binaer ausgeben
regsot:	jmp	regso_	; (Regs) ausgeben
       	.dw	0h		; Res.
;
; 
mentab:	.ds	0h		; Menue-Sprungadressen
		.dw	0f000h	; User-Funktion (@)
		.dw	6000h		; Assembler
		.dw	4000h		; BASIC
		.dw	copy_		; COPY
		.dw	6c00h		; Disassembler
		.dw	7000h		; Editor
		.dw	2000h		; FDC-Routinen
		.dw	go		; GO
		.dw	help		; HELP
		.dw	casin		; PROMO-CASIN
		.dw	jback		; Jump Back PROMO
		.dw	kill		; KILL
		.dw	list		; LIST
		.dw	mem		; MEM insp./modif./fill
		.dw	hrdcpy	; Hardcopy
		.dw	casot		; PROMO-CASOT
		.dw	prm		; Prommer
		.dw	sample	; Sample   
		.dw	compar	; Revise (COMPAR)
		.dw	step		; STEP
		.dw	txt		; TEXT
		.dw	ezu		; Uhr
		.dw	vvot		; V.24 Out
		.dw	vvin		; V.24 In
		.dw	excha_	; Exchange
		.dw	byte		; BYTE
		.dw	brkpt		; Breakpoint
;
;*************************************************
;      MAIN: ASCMO, CI/CSTS, CO, RAM, Peripherie
;
;                                                *
; ASCMO  MOPPEL-ASCII-Monitor  V 7.5    13.11.85 *
;                                                *
;*************************************************
;
;
;      Monitor fuer Bildschirmbetrieb und ASCII-
;      Tastatur; Verzweigung zu den verschie-
;      denen Routinen ueber Sprungtabelle MENTAB
;      nach Eingabe von "Return".
;
;
;***************************************************
;
;      Hauptprogrammschleife
;
;***************************************************
;
;      Parameter-Eingabe A...Z (auch a...z moeglich),
;      getrennt durch ein Blank (kann entfallen) und
;      gefolgt von maximal drei (bis zu vierstelli-
;      gen) HEX-Parametern.
;
;      Beispiel: C 2e,d00,29af
;                :  :   :   :..Zieladresse
;                :  :   :......Endadresse
;                :  :..........Startadresse
;                :...Befehl "C" (COPY=kopieren)
;
;      Anstelle von A...Z kann der Anwender-Parame-
;      ter "@" eingegeben werden (ebenfalls mit
;      maximal drei HEX-Parametern); nach "Return"
;      erfolgt in diesem Fall der Sprung zur
;      USER-Adresse mit Uebergabe der Parameter in
;      den Zellen SRCBEG, SRCEND und DSTBEG
;
;      Fehlermeldungen:0=Parameter fehlt
;                      1=zu viele Parameter
;                      2=Zeichenfehler (kein HEX)
;                      3=unerlaubter Parameter
;
;
;
kalt_:	lxi	sp,stack	; Kaltstart
		call	init
mokalt:	lxi	h,text0	; "Video-Monitor..."
		call	string
warm_:		xra	a	; Hilfsbank 0 einstellen
		out	fdctrl	; (d.h. FDC-Software)
mowarm:	lxi	sp,stack
       	lxi	h,text1	; Input-Zeichen
       	call	string
       	call	crson
moni:		call	ci		; Taste einlesen
       	mov	c,a		; ]bergabe an CO
       	call	co		; Echo ausgeben
       	cpi	1bh		; CTL+C?
       	jz	mokalt
       	lxi	h,moret	; Ret-Adresse
       	push	h		; in den Stack
       	cpi	0dh		; Return?
       	jnz	mocte		; kein Ret: JMP
       	call	crsoff
       	jmp	proces	; Verarbeitung
mocte:	pop	h
       	jmp	moni
;
moret: 	cz   error		; bei Err: Fehlerroutine
       	jz   mowarm		; ohne "o.k." weiter
       	lxi  h,text2	; "o.k."
       	call string
       	jmp  mowarm
;
;****************************************************
;
;      Initialisierung
;
;****************************************************
;
init:		mvi	a,rstoff	; RSTs sperren
		sim
		nop			; wg. NSC800-JR-Befehl
		mvi	a,0h		; CI-FLAG loeschen
		sta	ciflag
		lxi	h,0000h	; FDC-Spurregs. undefiniert
		shld	track0
		shld	track2
		call	loinit	; Drucker voreinstellen
		call	clrvid	; Bildschirm loeschen
;	
;      Video-Controller initialisieren
;
;      In der VIDLOP-Schleife wird jeweils eine
;      6845-Register-Adresse ausgegeben, gefolgt
;      vom zugehoerigen Datenbyte aus der Tabelle.
;
		mvi	c,8h		; Tabellenlaenge
		lxi	d,hordsp	; Tabellenanfang
		lxi	h,bigtab
		in 	vidctl	; Formatwahl
		ani	80h		; MSB maskieren
		jnz	bigcte	; Grossform.:JMP
		lxi	h,smltab
bigcte:	mov	a,m		; Tabellenwert
		stax	d		; ablegen
		inx 	h
		inx 	d
		dcr 	c
		jnz 	bigcte	; Schleife
		lxi	b,0eh		; B=00,C=14d
vidlop:	mov	a,b		; Reg-Nr.
		out	vidreg	; Reg-Nr.
		mov	a,m		; Tabellenwert
		out	vidat		; Reg-Daten
		inx	h
		inr	b
		dcr	c
		jnz	vidlop
		lhld	linz
		shld	update
		shld	0h		; Res.
		ret
;
; Tabellen fr die Bildschirmdarstellung 80/40-Zeichen pro Zeile
;
bigtab:	.ds	0h		; Tab.Grossformat
       	.db	50h		; 80d Zeichen/Zeile
       	.db	18h		; 24d Zeilen/Seite
       	.dw	3050h		; Beginn Zeile 2
       	.dw	36e0h		; Beg.vorletzte Zeile
       	.dw	3730h		; Beg.letzte Zeile	
       	.db	71h		; hortot=114-1
       	.db	50h		; hordsp
       	.db	59h		; horsyn
       	.db	69h		; vhpuls
       	.db	19h		; vertot=26-1
       	.db	20h		; veradj
       	.db	18h		; verdsp
       	.db	18h		; versyn
       	.db	30h		; mode
       	.db	0bh		; lines=12-1
       	.db	6ah		; crssta
       	.db	0bh		; crsend
       	.db	0h		
       	.db	0h		
;
smltab:	.ds	0h   		; Tab. Kleinformat
       	.db	28h  		; 40d Zeichen/Zeile
       	.db	12h  		; 18d Zeilen/Seite
       	.dw	3028h		; Beginn Zeile 2
       	.dw	3280h		; Beg.vorletzte Zeile
       	.dw	32a8h		; Beg.letzte Zeile
; 	
       	.db	38h		; hortot=57-1
       	.db	28h		; hordsp
       	.db	2ch		; horsyn
       	.db	25h		; vhpuls
       	.db	19h		; vertot=26-1
       	.db	01h		; veradj
       	.db	12h		; verdsp
       	.db	15h		; versyn
       	.db	30h		; mode
       	.db	0bh		; lines=12-1
       	.db	6ah		; crssta
       	.db	0bh		; crsend
       	.db	0h		
       	.db	0h		
;
;***************************************************
;
; Verarbeitung der Eingabezeile
;
;***************************************************

;
proces:	lhld	linz		; Eingabezeile
		lxi	d,12h		; max. 3 Vierstellige Param.
		dad	d		; H&L auf Blank hinter Param.
		mvi	c,10h		; Laenge ohne "M="
		call	parin_	; Parameter einlesen
		rz			; fehlerhafte Eingabe
projmp:	ani	5fh		; Grossbuchstaben erzeugen
		cpi	5bh		; "Z"+1
		jnc	inperr
		cpi	40h		; "@" (=User)
		jnc	procte
inperr:	xra	a		; FLAGs setzen
		mvi	a,13h		; Input nicht im Menue
		ret			; Fehler: Z=1
procte:	sui	40h		; Versatz erzeugen
		rlc	
		mov	e,a
		lxi	h,mentab	; Menue-Sprungadressen
		dad	d
		mov	e,m		; Zieladresse (lower)
		inx	h
		mov	d,m		; Zieladresse (upper)
		xchg			; H&L=Zieladresse
		pchl			; Verzweigung
		nop
;
;***************************************************
;
;      PARIN  Parameter vom Bildschirm einlesen
;
;***************************************************
;
;  Einsprung: H&L zeigen auf Blank hinter Parameter
;             REG C haelt Parameter-Laenge
;
;  Ruecksprung:Z=1 heisst Fehler (Fehlercode in REG A)
;             sonst stehen die Parameter 1,2 und 3
;             in den Zellen SRCBEG,SRCEND u. DSTBEG
;
parin_:	push	h
		lxi 	h,0h		; Parameter-Buffer loeschen
		shld	srcbeg
		shld	srcend
		shld	dstbeg
		pop 	h
		mov 	a,m		; Zeichen vom Bildschirm holen
		cpi 	20h		; Blank?
		jz  	inloop	; Blank=o.k, d.h. JMP
		xra 	a		; FLAGs setzen
		mvi 	a,11h		; String zu lang
		ret			; Fehler: Z=1
; 
inloop:	lxi	d,0h		; Displacement
		xchg
		shld	dig1		; Hilfs-Buffer loeschen
		shld	dig3
		xchg
inlop: 	dcx	h
		dcr	c
		jp 	inlop2
		xra	a		; FLAGs setzen
		mvi	a,10h		; Input fehlt
		ret			; Fehler: Z=1
inlop2:	mov	a,m
		cpi	20h		; Blank?
		jz 	inlop		; ja:JMP (nachruecken)
		mov	b,a		; Zeichen retten
		xra	a		; REG A loeschen
		cmp	c		; Param.-Anfang erreicht?
		mov	a,b
		jnz	incte		; nein: JMP
		call	assemb	; Digits zusammensetzen
		inr	c		; FLAGs loeschen
parend:	ret
;
incte: 	cpi  2ch		; Komma?
       	jz   inct		; ja: JMP
       	cpi  2eh		; Punkt?
       	jz   inct		; ja: JMP
       	call hxtst		; Input ueberpruefen
       	rz			; Fehler: Z=1
       	jmp  inlop		; sonst weiter
; 
inct:  	call assemb    	; Digits zusammensetzen
       	jmp  inloop
;
assemb:	push	psw
       	push	h
       	push	d
       	lxi 	h,dig1	; Digits zu Doppelwort
       	mov 	e,m		; zusammensetzen
       	inx 	h
       	mov 	a,m
       	rlc			; oberes Nibble im
       	rlc			; unteren Wort
       	rlc	
       	rlc	
       	ora 	e		; unteres Wort kpl.
       	mov 	e,a
       	inx 	h
       	mov 	d,m
       	inx 	h
       	mov 	a,m
       	rlc			; oberes Nibble im
       	rlc			; oberen Wort
       	rlc	
       	rlc	
       	ora 	d		; oberes Wort kpl.
       	mov 	d,a
;
		lhld	srcend	; Parameter nachschieben
		shld	dstbeg
		lhld	srcbeg
		shld	srcend
		xchg	
		shld	srcbeg
		pop 	d
		pop	h
		pop	psw
		ret
;
hxtes_:	lxi	d,0h		; f.externen Einsprung
hxtst:	ani	5fh		; Grossbuchstabe
		cpi	10h		; "0"
		jc 	hexerr	; Code zu klein:JMP
		cpi	1ah		; "9"+1
		jc 	hexcte	; Code o.k.:JMP
		adi	0c9h		; A...F adjust
		cpi	10h		; "F"+1
		jnc	hexerr	; Code zu gross:JMP
		cpi	0ah		; korr. "A"
		jnc	insft		; A...F: JMP
hexerr:	xra	a		; FLAGs setzen
		mvi	a,12h		; kein HEX-Digit
		ret			; Fehler: Z=1
hexcte:	sui	10h		; Rest der "3x"
insft:	push	h		; Param. einsammeln
		lxi	h,dig1	; Parameter-Buffer
		dad	d		; Displacement
		mov	m,a
		inx	d
		pop	h
		mov	b,a
		xra	a
		inr	a		; FLAGs loeschen
		mov	a,b
		ret			; REG A: HEX-Nibble
;		
text0:	.db	0dh		; Ret
		.text	"Moppel-Video-Monitor"
		.db	20h
		.db	"V"
		.db	20h
		.db	"7"
		.db	"."
		.db	"5"
		.db	0dh		; Ret
		.text	"Copyright"
		.db	20h
		.text	"(C)"
		.db	20h
		.db	0eh
		.db	0fh
		.db	10h
		.db	27h
		.text	"85"
		.db   0dh		; Ret
		.db   0dh		; Ret
		.text	"Anweisung"
		.db	20h
		.db	"a"
		.db	2eh
		.db	2eh
		.db	2eh
		.db	"Z"
		.db	20h
		.text	"eingeben"
		.db	3ah
		.db	0h
;		
text1:	.db	0dh		; Ret
		.db	0dh		; Ret
		.db	"M"
		.db	3eh
		.db	0h
;			
text2:	.db	0dh		; Ret
		.db	"o"
		.db	2eh
		.db	"k"
		.db	2eh
		.db	0h
;
;***************************************************
;
; Zeichenvorrat darstellen
;
;***************************************************
;
sampl_:	call	clrvid	; clear screen
		lda	verdsp
		sui	4h
		rrc			; wg. Leerzeile
		ani	0fh
		mov	b,a		; Zeilenzaehler
		mvi	c,0ffh	; Code-Zaehler
;		
samlop:	lda	hordsp	; Zeichenzaehler
		rrc			; wg.Zwischenraum
		ani	0f0h		; 4 LSB ausblenden
samlp:	push	psw
		mov	a,c
		inr	a		; naechster Code
		ani	7fh		; Invers-Bit unterdruecken
		mov	c,a
		pop	psw
		call	co2		; Character ausgeben
		dcr	a
		jnz	samlp
		call	crlf
		call	crlf
		dcr	b
		jnz	samlop
		jmp	warm
;
;***************************************************
;
; Fehlermeldung erzeugen
;
;***************************************************
;
error_:	push	psw		; Err-Code retten
		lxi 	h,errmsg
		call	string
		pop 	psw
		push	psw
		rrc			; oberes Nibble
		rrc	
		rrc	
		rrc	
		ani 	7h
		mov 	e,a		; Prefix 0...7
		mvi 	d,0h
		lxi 	h,errtab
		dad 	d
		mov 	c,m		; Buchstaben auslesen
		call	co
		mvi 	c,30h		; Null ausgeben
		call	co		; Code-Nr. (HI)
		pop 	psw
		ani 	0fh		; unteres Nibble
		ori 	30h		; ASCII-Zahl
		mov 	c,a
		call	co		; Code-Nr. (LO)
		call	crlf
		out 	belout	; akustisches Signal
		xra 	a		; Err-Kennzeichen
		ret	
;		
errmsg:	.db	0dh		; CRLF
		.db	0dh
		.db	"E"
		.db	"r"
		.db	"r"
		.db	"o"
		.db	"r"
		.db	20h		; Blank
		.db	23h		; #
		.db	20h		; Blank
		.db	0h
		.dw	0h		; Res.
; 	
errtab:	.db	"H"		; HEX-Monitor
       	.db	"M"		; MOVID
       	.db	"F"		; Floppy
       	.db	"P"		; Printer
       	.db	"B"		; BASIC
       	.db	"D"		; Disassembler
       	.db	"A"		; Assembler
       	.db	"E"		; Editor
;
;***************************************************
;
; Ausgabe von Registerinhalten (ab UPDATE)
;
;***************************************************
;
bytbi_:	mvi	c,8h		; Bitzaehler
bytlop:	rlc			; MSB ins CY-Bit
		push	psw
		mvi	a,0h		; Acc loeschen
		jnc	bytcte	; LOW-Bit:JMP
		inr	a		; HIGH-Bit
bytcte:	push	b
		call	nibot
		pop	b
		pop	psw
		dcr	c
		jnz	bytlop
		ret	
;
nibot: 	ani	0fh		; lower Nibble maskieren
		adi	30h		; waere ASCII-Ziffer
		cpi	3ah		; "9"+1
		jc 	nibcte	; "0"..."9":JMP
		adi	7h		; "A"..."F"
nibcte:	mov	c,a		; Zeichen ausgeben
		call co
		ret
;
bytspc:	call	space
bytot_:	push	psw
		rrc	
		rrc	
		rrc	
		rrc	
		call	nibot
		pop 	psw
		call	nibot
		ret
;
adrspc:	call	space
		jmp 	adro
adrot_:	xchg	
adro:		push	d
		mov 	a,d
		call	bytot_
		pop 	d
		mov 	a,e
		call	bytot_
		ret
;
regso_:	lda 	rega		; REG A ausgeben
		call	bytspc
		lhld	regb		; REG B&C ausgeben
		mov 	e,h
		mov 	d,l
		call	adrspc
		lhld	regd		; REG D&E ausgeben
		mov 	e,h
		mov 	d,l
		call	adrspc
		lhld	regh		; REG H&L ausgeben
		mov 	e,h
		mov 	d,l
		call	adrspc
		lhld	regs		; SP ausgeben
		mov 	e,h
		mov 	d,l
		call	adrspc
		call	space		; Zwischenraum
		lda 	regf		; FLAGs ausgeben
		call	bytbi_
		ret
;
co2:		push	psw
		mov 	a,c
		cpi 	08h		; "Bell" +1
		jc  	cocte		; kleiner: JMP
		cpi 	0eh		; "CR"  +1
		jnc 	cocte		; nicht kleiner: JMP
		pop 	psw
		call	space		; Codes 7...Dh unterdruecken
		jmp 	space
cocte:	pop 	psw
		call	co
; 
crlf:		push	b
		mvi 	c,0dh		; Wagenruecklauf
		call	co
		mvi 	c,0ah		; Zeilenvorschub
		call	co
		pop 	b
		ret
;
;
;*************************************************
;						 		 ;
;                                                ;
; CI/CSTS  Console-Eingabe, -Status     13.11.85 ;
;                                                ;
;*************************************************
;
;      Verwaltung mittels zweier Unterprogramme:
;
;      CI   (Console Input) wartet auf Tastendruck,
;           entprellt, liest Zeichen ein und ueber-
;           gibt es in REG A.
;
;      CSTS (Console Status) meldet Tastatur-Status
;           key:   (REG A)=FFh, ZERO-FLAG off (Z=0)
;                  (REG D)=Input 80h...01h
;                  (REG E)=Zeilen-Nr. 0...Fh
;           no key:(REG A)=00h, ZERO-FLAG on (Z=1)
;
;***************************************************
;
;      Einlese-Unterprogramm
;
;***************************************************
;
ci_:		jmp	ci2
cipar:	jmp	cip		; ASCII parallel
ci2:		push	h
		push	d
		push	b
		lda	ciflag
		ora	a
		jz 	cionlp	; CI-FLAG off: JMP
cioflp:	call	csts_ 	; auf Loslassen warten
		jnz	cioflp	
cionlp:	call	csts_ 	; auf Tastendruck warten
		jz	cionlp	
cict:		lda 	timprl	; Prellzeit in ms
prlop: 	call	dely1	
		dcr 	a	
		jnz 	prlop	
		call	csts_ 	; Tastenmatrix abfragen
		lhld	hibuf 	; Inputs holen
		xchg	      	; in D&E uebergeben
		mvi 	a,0fh	
		sta 	ciflag	; FLAG setzen (CSTS loescht es)
		mov 	a,d   	; Input ins REG A
		ani 	1fh   	; CTL-Keys ausblenden
		jz  	endci 	; keine anderen Keys: JMP
		push	psw   	; modif. Input retten
		mov 	a,e	
		rlc		
		rlc 	      	; Zeilen-Nr. ; 4
		mov 	e,a	
		pop 	psw	
inrci: 	rrc 	      	; Spalten-Nr. binaer ums.
		jc  	cicte	
		inr 	e	
		jmp 	inrci	
cicte: 	mov 	a,d	
		ani 	10h   	; Cursor-Taste?
		jnz 	cursor	; ja: JMP
		mov 	a,d	
		ani 	20h   	; FCT-Taste?
		jnz 	funct 	; ja: JMP
		mov 	a,d	
		ani 	0c0h  	; SFT/CTL-Keys maskieren
		mov 	b,a   	; REG B: Basis-Versatz
		cpi 	0c0h  	; beide gleichz.aktiv?
		mvi 	a,0h  	;  =NUL
		jz  	endci 	; ja: JMP
;
getcod:	lda	ioflg 	; Shift lock abfragen
		ani	1h    	; LSB gesetzt
		jz 	getct 	; nein: JMP
		mvi	b,40h 	; SHIFT-Versatz
;		
getct: 	lxi	h,0h  	; CP/M-Abfrage
		dad	sp	
		mvi	a,0fah	; SP=FAxx?
		cmp	h	
		jz 	getcpm	; CP/M: JMP
		mvi	a,7ah 	; SP=7Axx?
		cmp	h	
		jz 	getcpm	; CP/M: JMP
		mvi	a,29h 	; SP=29xx?
		cmp	h	
		jz 	getcpm	; CP/M: JMP
		xra	a     	; SP=0080/0100?
		cmp	h	
		jnz	getcte	; CP/M: JMP
getcpm:	mvi	a,80h 	; CTL-Codes in CP/M?
		cmp	b	
		jnz	getcte	; nein: JMP
		mvi	b,0c0h	; CP/M-CTL-Versatz
getcte:	xra	a     	; REG A loeschen
		add	b     	; Basis-Versatz addieren
		add	e     	; Key-Nr. addieren
		mov	e,a	
		mvi	d,0h	
		lxi	h,asctab
		dad	d		; H&L: Tabellenadresse
		mov	a,m		; ASCII-Code auslesen
endci: 	pop	b
		pop	d
		pop	h
		ret	
;
;      CURSOR    erzeugt die Cursor-Steuercodes
;
;      80h: links    (CTL+H ./. 08h = BS)
;      82h: rechts   (CTL+J ./. 0Ah = LF)
;      89h: auf      (CTL+U ./. 15h = NAK)
;      8Bh: ab       (CTL+N ./. 0Eh = SO)
;      8Fh: home     (CTL+Z ./. 1Ah = SUB)
;
;      88h: llinks    (CTL+DEL)
;      8Ah: rrechts   (CTL+TAB)
;
crs_:		mov	a,e		; Zeilen-Nr. ; 4
		ani	3ch		; Spalten-Anteil loeschen
		rrc	
		rrc	
		dcr	a		; Zeilen-Nr. 0...F
		mov	e,a
		mvi	d,0h
		lxi	h,crstab	; Code-Tabelle
		dad	d
		mov	a,m		; ASCII-Code auslesen
		jmp	endci
;
crstab:	.db	8fh		; HOME/F
		.db	2bh		; +/REG
		.db	2ah		; ; /DAT
		.db	2eh		; ./BST
		.db	3fh		; ?/RUN
		.db	2fh		; ./. /D
		.db	2dh		; -/ADR
		.db	8fh		; HOME (Ext)
		.db	8bh		; down (Ext)
		.db	80h		; left/C;  auch Ext
		.db	82h		; right (Ext)
		.db	0ffh		; n.c.
		.db	8bh		; down/B
		.db	82h		; right/A
		.db	89h		; up/E
		.db	89h		; up (Ext)
;
;
; FUNCT	verzweigt zu Sprungzielen, die
; 		im FCTBUF im RAM abgelegt sind
; 		(erst untere, dann obere Adress-
; 		haelfte); Aufruf durch gleich-
; 		zeitiges Druecken der (gruenen)
; 		Funktionstaste und einer der
; 		oben (!) liegenden Zifferntasten
; 		0...9 (bei FCT+7 wird zu derjenigen
; 		Adresse verzweigt, die in den
; 		RAM-Zellen 2E20/21h abgelegt
; 		ist usf; vgl.Basis-Symboltabelle)
; 		Die vier Tasten einer Zeile sind
; 		gleichberechtigt, d.h. "FCT+0" be-
; 		wirkt dieselbe Reaktion wie "FCT+\"
;
funct:	mov	a,e		; REG E: 1Ch...3Fh
		ani	3ch
		rrc	
		rrc			; alte Zeilennummer
		mov	e,a
		mvi	d,0h
		lxi	h,fctbuf	; Zieladressen im RAM
		dad	d
		mov	e,m
		inx	h
		mov	d,m
		xchg
		pchl			; H&L: Sprungziel
;
;***************************************************
;
;     Abfrage-Unterprogramm
;	mit Hilfsroutine da IN nicht indirekt 
;	adressiert werden kann
;
;***************************************************
;
csts_:	jmp 	csts2
cstspa:	jmp 	cstsp		; ASCII parallel
csts2:	push	h
		push	d
		push	b
		lxi 	h,hibuf	; Input-Hilfsroutine
		mvi 	m,0dbh	; "IN"
		inx 	h
		inx 	h
		mvi 	m,0c9h	; "RET"
		dcx 	h		; H&L: RAM-Zelle f.Portadr.
		mvi 	m,0fch	; oberste Key-Portadresse
		mvi 	c,0fh		; Zeilenzaehler
cstlop:	call	hibuf		; Key-Port einlesen
		push	h
		lxi 	h,keybyt	; und bei Bedarf
		xra 	m		; invertieren
		pop 	h
		mov 	d,a		; REG D: Input-Daten
		ani 	1fh		; 3 MSB ausblenden
		jz  	dcradr	; no key: JMP
		mov 	e,c		; Key: Zaehlerstand retten
		jmp 	cstend	; und zurueckspringen
dcradr:	mov 	a,m		; Portadresse holen
		sui 	10h		; erniedrigen
		mov 	m,a		; und wieder ablegen
		dcr 	c		; Zeilenzaehler erniedrigen
		jp  	cstlop	; nicht fertig: JMP
cstend:	mov 	a,d		; Input auf 00 abfragen
		ani 	1fh		; Input-Daten
		jz  	csret		; no Key: (REG A)=00, Z=1
		mvi 	a,0ffh	; Key:    (REG A)=FF, Z=0
		xchg			; D&E fuer CI retten
		shld	hibuf
csret:	pop 	b
		pop 	d
		pop 	h
		push	psw
		mvi 	a,0h
		sta 	ciflag	; CI-FLAG loeschen
		pop 	psw
		ret
;
;
;****************************************************
;
;      ASCII-Paralleleingabe (Standard-Tastatur)
;
;****************************************************
;
;  Anschluss der Tastatur ueber einen 8-Bit-Parallel-
;  Port (STROBE auf Bit 7); gewaehlte Portadresse:A8h
;
asport	.equ	0a8h		; z.B. Parallel-Interface
;
cip:		push	h
		push	d
		push	b
		lda 	ciflag
		ora 	a
		jz  	cponlp	; CI-FLAG off: JMP
cpoflp:	call	cstsp		; auf Loslassen warten
		jnz 	cpoflp
cponlp:	call	cstsp		; auf Tastendruck warten
		jz  	cponlp
		mvi 	a,0fh
		sta 	ciflag	; FLAG setzen (CSTS loescht es)
		in  	asport	; Code einlesen
		pop 	b
		pop 	d
		pop 	h
		ret	
;
cstsp:	mvi	a,0h
		sta	ciflag	; CI-FLAG loeschen
		in 	asport	; STROBE einlesen
		nop			; evtl. invertieren
		ani	80h		; MSB maskieren
		rz			; no key: ACC=0; Z=on
		mvi	a,0ffh
		ret			; key: ACC=FF; Z=off
;
;****************************************************
;
;      Tabelle der ASCII-Codes
;
;****************************************************
;
;      Normal Codes
;
asctab:	.ds	0h
; 0
		.db	37h		; 7
		.db	6ah		; j
		.db	6dh		; m
		.db	75h		; u
; 1			
		.db	33h		; 3
		.db	64h		; d
		.db	63h		; c
		.db	65h		; e
; 2			
		.db	35h		; 5
		.db	67h		; g
		.db	62h		; b
		.db	74h		; t
; 3			
		.db	31h		; 1
		.db	61h		; a
		.db	79h		; y
		.db	71h		; q
; 4			
		.db	36h		; 6
		.db	68h		; h
		.db	6eh		; n
		.db	7ah		; z
; 5			
		.db	32h		; 2
		.db	73h		; s
		.db	78h		; x
		.db	77h		; w
; 6			
		.db	34h		; 4
		.db	66h		; f
		.db	76h		; v
		.db	72h		; r
; 7+			
		.db	39h		; 9
		.db	33h		; 3
		.db	0dh		; Return
		.db	36h		; 6
; 8+			
		.db	37h		; 7
		.db	31h		; 1
		.db	30h		; 0
		.db	34h		; 4
; 9			
		.db	81h		; Delete (PROMO)
		.db	0dh		; Return
		.db	0ff		; Shift lock
		.db	09h		; Horizontal Tabulation
; A			
		.db	2bh		; +
		.db	23h		; #
		.db	0h 		; HALT
		.db	3ch		; "less than"
; B				
		.db	7eh		; ss
		.db	7bh		; ae
		.db	20h		; "Blank"
		.db	7dh		; ue
; C			
		.db	30h		; 0
		.db	7ch		; oe
		.db	2dh		; -
		.db	70h		; p
; D			
		.db	39h		; 9
		.db	6ch		; l
		.db	2eh		; .
		.db	6fh		; o
; E			
		.db	38h		; 8
		.db	6bh		; k
		.db	2ch		; ,
		.db	69h		; i
; F+			
		.db	38h		; 8
		.db	32h		; 2
		.db	2eh		; .
		.db	35h		; 5
;		
;      Shift Codes
;
; 0
		.db	2fh		; /
		.db	4ah		; J
		.db	4dh		; M
		.db	55h		; U
; 1					
		.db	40h		; @
		.db	44h		; D
		.db	43h		; C
		.db	45h		; E
; 2					
		.db	25h		; %
		.db	47h		; G
		.db	42h		; B
		.db	54h		; T
; 3					
		.db	21h		; !
		.db	41h		; A
		.db	59h		; Y
		.db	51h		; Q
; 4					
		.db	26h		; &
		.db	48h		; H
		.db	4eh		; N
		.db	5ah		; Z
; 5					
		.db	22h		; "
		.db	53h		; S
		.db	58h		; X
		.db	57h		; W
; 6					
		.db	24h		; $
		.db	46h		; F
		.db	56h		; V
		.db	52h		; R
; 7+					
		.db	39h		; 9
		.db	33h		; 3
		.db	0dh		; Return
		.db	36h		; 6
; 8+					
		.db	37h		; 7
		.db	31h		; 1
		.db	30h		; 0
		.db	34h		; 4
; 9
		.db	08h		; Delete (ASCII BS)
		.db	0dh		; Return
		.db	0ffh		; Shift lock
		.db	09h		; Horizontal Tabulation
; A					
		.db	2ah		; "Asterisk"
		.db	5eh		; "Circumflex"
		.db	0h 		; HALT
		.db	3eh		; "greater than"
; B					
		.db	3fh		; ?
		.db	5bh		; [
		.db	20h		; "Blank"
		.db	5dh		; ]
; C					
		.db	3dh		; =
		.db	5ch		; \
		.db	5fh		; "Underline"
		.db	50h		; P
; D					
		.db	29h		; )
		.db	4ch		; L
		.db	3ah		; :
		.db	4fh		; O
; E					
		.db	28h		; (
		.db	4bh		; K
		.db	3bh		; ; 
		.db	49h		; I
; F+					
		.db	38h		; 8
		.db	32h		; 2
		.db	2eh		; .
		.db	35h		; 5
;
;      Control Codes (MOPPEL)
;
; 0
		.db	0b7h		; 7;  Wurzel
		.db	82h 		; j;  Cursor rechts
		.db	0a8h		; m;  griech. "m"
		.db	89h 		; u;  Cursor up
; 1			
		.db	0b3h		; 3;  3 Exp.
		.db	0a4h		; d;  griech. "d"
		.db	1bh 		; c;  ESC (BREAK)
		.db	0a5h		; e;  griech. "e"
; 2			
		.db	0b5h		; 5;  Winkel
		.db	0a3h		; g;  griech. "g"
		.db	0a2h		; b;  griech. "b"
		.db	0abh		; t;  griech. "t"
; 3			
		.db	0b1h		; 1;  1/
		.db	0a1h		; a;  griech. "a"
		.db	0eh 		; y;  SO
		.db	11h 		; q;  DC1
; 4			
		.db	0b6h		; 6;  Promille
		.db	80h 		; h;  Cursor links
		.db	8bh 		; n;  Cursor down
		.db	8fh 		; z;  Cursor home
; 5			
		.db	0b2h		; 2;  2 Exp.
		.db	0aah		; s;  griech. "s"
		.db	15h 		; x;  NAK
		.db	13h 		; w;  DC3
; 6			
		.db	0b4h		; 4;  +/-
		.db	90h 		; f;  Apostroph
		.db	0fh 		; v;  SI
		.db	07h 		; r;  Bell
; 7+			
		.db	0f9h		; CTL+9
		.db	0f3h		; CTL+3
		.db	0dh 		; Return
		.db	0f6h		; CTL+6
; 8+			
		.db	0f7h		; CTL+7
		.db	0f1h		; CTL+1
		.db	0f0h		; CTL+0
		.db	0f4h		; CTL+4
; 9			
		.db	88h 		; CTL+DEL (=LLEFT)
		.db	0dh 		; Return
		.db	0ffh		; Shift Lock
		.db	8ah 		; CTL+TAB (=RRIGHT)
; A
		.db	0bch		; +;  eck.Kl. auf
		.db	0bdh		; #;  eck.Kl. zu
		.db	0h  		; HALT
		.db	0beh		; less than;  geschw.Kl.auf
; B			
		.db	0bfh		; ss;  geschw.Kl.zu
		.db	0a0h		; ae;  At Sign
		.db	20h 		; Blank
		.db	0bbh		; ue;  Fuesse oben
; C			
		.db	0b0h		; 0;  fettes "s"
		.db	0bah		; oe;  Fuesse unten
		.db	0afh		; -;  fettes "m"
		.db	0a9h		; p;  griech. "p"
; D			
		.db	0b9h		; 9;  Strich
		.db	0a7h		; l;  griech. "l"
		.db	0aeh		; .;  fettes "h"
		.db	0ach		; o;  griech. "o"
; E			
		.db	0b8h		; 8;  ungefaehr
		.db	0a6h		; k;  griech. "k"
		.db	0adh		; ,;  Grad
		.db	91h 		; i;  Gegen-Apostroph
; F+			
		.db	0f8h		; CTL+8
		.db	0f2h		; CTL+2
		.db	0feh		; CTL+"."
		.db	0f5h		; CTL+5
; 
;      Control Codes (CP/M)
;
; 0
		.db	60h		; 7;  Spare
		.db	0ah		; J;  Line Feed
		.db	0dh		; M;  Return
		.db	15h		; U;  NAK
; 1			
		.db	1dh		; 3;  GS
		.db	04h		; D;  End of Transmission
		.db	03h		; C;  ETX
		.db	05h		; E;  Enquiry
; 2			
		.db	1fh		; 5;  US
		.db	07h		; G;  Bell
		.db	02h		; B;  Start of Text
		.db	14h		; T;  Device Control 4
; 3			
		.db	1Bh		; 1;  ESC
		.db	01h		; A;  Start of Heading
		.db	19h		; Y;  End of Medium
		.db	11h		; Q  Device Control 1
; 4			
		.db	27h		; 6;  Spare
		.db	08h		; H;  BS
		.db	0eh		; N;  SO
		.db	1ah		; Z;  SUB
; 5			
		.db	1ch		; 2;  FS
		.db	13h		; S;  Device Control 3
		.db	18h		; X;  Cancel
		.db	17h		; W;  End of Transm. Block
; 6			
		.db	1eh		; 4;  RS
		.db	06h		; F;  Acknowledge
		.db	16h		; V;  Synchronous idle
		.db	12h		; R;  Device Control 2
; 7+			
		.db	7fh		; 9;  Spare
		.db	1dh		; 3;  GS
		.db	0dh		; Return
		.db	27h		; 6;  Spare
; 8+	
		.db	60h 		; 7;  Spare
		.db	1bh 		; 1;  ESC
		.db	00h 		; 0;  (NUL)
		.db	1eh 		; 4;  RS
; 9			
		.db	7fh 		; Delete (ASCII DEL)
		.db	0dh 		; Return
		.db	0ffh		; Shift Lock
		.db	00h 		; TAB;  (NUL)
; A			
		.db	00h 		; +;  (NUL)
		.db	00h 		; #;  (NUL)
		.ds	1h  		; Spare Key
		.db	00h 		; greater;  (NUL)
; B			
		.db	00h 		; ss;  (NUL)
		.db	00h 		; [;  (NUL)
		.db	00h 		; Blank;  (NUL)
		.db	00h 		; ];  (NUL)
; C			
		.db	00h 		; 0;  (NUL)
		.db	00h 		; \;  (NUL)
		.db	00h 		; -;  (NUL)
		.db	10h 		; P;  Data Link escape
; D			
		.db	7fh 		; 9;  Spare
		.db	0ch 		; L;  Form feed
		.db	00h 		; .;  (NUL)
		.db	0fh 		; O;  Shift in
; E			
		.db	67h 		; 8;  Spare
		.db	0bh 		; K;  Vertical Tabulation
		.db	00h 		; ,;  (NUL)
		.db	09h 		; I;  Horizontal Tabulation
; F+			
		.db	67h 		; 8;  Spare
		.db	1ch 		; 2;  FS
		.db	00h 		; .;  (NUL)
		.db	1fh 		; 5; US
;
;
;***************************************************
;
;      Control- und Spare-Codes fuer CP/M
;
;***************************************************
;
;      00h: CTL+Space NUL  Null
;      01h: CTL+A     SOH  Start of Heading
;      02h: CTL+B     STX  Start of Text
;      03h: CTL+C     ETX  End of Text
;      04h: CTL+D     EOT  End of Transmission
;      05h: CTL+E     ENQ  Enquiry
;      06h: CTL+F     ACK  Acknowledge
;      07h: CTL+G     BEL  Bell
;      08h: CTL+H     BS   Backspace
;      09h: CTL+I     HT   Horizontal Tabulation
;      0Ah: CTL+J     LF   Line feed
;      0Bh: CTL+K     VT   Vertical Tabulation
;      0Ch: CTL+L     FF   Form feed
;      0Dh: CTL+M     CR   Carriage Return
;      0Eh: CTL+N     SO   Shift out
;      0Fh: C7L+O     SI   Shift in
;      10h: CTL+P     DLE  Data Link Escape
;      11h: CTL+Q     DC1  Device Control 1
;      12h: CTL+R     DC2  Device Control 2
;      13h: CTL+S     DC3  Device Control 3
;      14h: CTL+T     DC4  Device Control 4
;      15h: CTL+U     NAK  Negative Acknowledge
;      16h: CTL+V     SYN  Synchronous idle
;      17h: CTL+W     ETB  End of transm. Block
;      18h: CTL+X     CAN  Cancel
;      19h: CTL+Y     EM   End of Medium
;      1Ah: CTL+Z     SUB  Substitute Character
;
;      1Bh: CTL+1     ESC  Escape
;      1Ch: CTL+2     FS   File Separator
;      1Dh: CTL+3     GS   Group Separator
;      1Eh: CTL+4     RS   Record Separator
;      1Fh: CTL+5     US   Unit Separator
;
;      27h: CTL+6          "Apostroph"
;      60h: CTL+7          "Apostroph2"
;      67h: CTL+8          Spare
;      7fh: CTL+9     DEL  Delete
;
;*************************************************
;                                                *
; CO  Console-Ausgabe                   13.11.85 *
;                                                *
;*************************************************
;
;
;      Ausgabe von Zeichen ins Video-RAM zur
;      Darstellung auf dem Bildschirm; Paral-
;      leldruck wird vom I/O-FLAG gesteuert:
;
;      Aufbau des I/O-FLAGs (IOFLG):
;
;     Bit7 6 5 4 3 2 1 0
;        : : : : x x x :
;        : : : :       + - Shift Lock
;        : : : + - - - - - Thermo-Drucker
;        : : + - - - - - - 8255-         /  laeuft
;        : + - - - - - - - Centronics-   /  zu CO
;        + - - - - - - - - V.24-         /parallel
;
;
;      Das Umschalten der Bits 1...7 erfolgt
;      durch Betaetigen der Tasten CTL+1...7.
;
;
co_:		push	psw		;
		push	h
		push	d
		push	b
		mov	a,c
		lxi	h,coutct	; cte-Adresse
		push	h		; in den Stack
		cpi	7h		; Bell
		jz	bell
		cpi	8h		; BS
		jz	delet
		cpi	7fh		; Delete
		jz	delet
		cpi	81h		; PROMO-DEL
		jz	delet
		cpi	9h		; HT
		jz	tabhor
htab:		.equ	8h		; hor. TAB-Weite
		cpi	0ah		; LF
		jz	linfe_	; 
		cpi	0bh		; VT
		jz	tabver
vtab:		.equ	4h		; vert. TAB-Weite
		cpi	0ch   	; FF
		jz 	clrvi_	
		cpi	0dh   	; CR
		jz 	caret	
		cpi	0ffh  	; Shift Lock?
		jz 	shilok	
		ani	0f0h  	; 4 MSB maskieren
		cpi	80h   	; Cursor-Fct?
		jz 	cursor	
		cpi	0b0h  	; CTL-Codes
		jz 	contrl	
		cpi	0a0h  	; Sonderzeichen
		jnz	cout
		mov	a,c
		ani	1fh
		mov	c,a
cout:		pop	h		; cte-Adr. loeschen
		lhld	update	
blk1:		in	vidctl	; Blanking abwarten
		rrc		
		jnc	blk1	
		mov	m,c   	; Zeichen ausgeben
		inx	h	
		inx	h     	; wg. RAMAX-Limit +1
		call	ramax 	; ausserhalb?
		dcx	h	
		jc	coutct	; nein: JMP
		call	crlf_ 	; Zeilenvorschub
coutct:	call	crspo_	
; 		
coprt:	lda 	ioflg	
		ani 	0f0h  	; 4 MSB maskieren
		jz  	coend 	; kein Paralleldruck:JMP
		mvi 	a,0eh 	; (h)
		cmp 	c	
		jnz 	coprt2	
		mvi 	c,68h		; "h"
		jmp 	colo	
coprt2:	inr 	a     	; (m)
		cmp 	c	
		jnz 	coprt3	
		mvi 	c,6dh		;  "m"
		jmp 	colo	
coprt3:	inr 	a     	; (s)
		cmp 	c	
		jnz 	colo
		mvi 	c,73h		;  "s"
colo:		call	lo		; Drucker-Routine
		push	psw		
		cz 	error 	; ev. Fehler melden
		pop	psw	
		cpi	0dh   	; Return?
		jnz	coend 	; nein: JMP
		mvi	c,0ah 	; fuer explizites LF
		jmp	coend 	; (dann hier CALL LO)
coend:	pop	b	
		pop	d	
		pop	h	
		pop	psw	
		ret		
;
bell:		lhld	update	
		out	belout	
		ret	
;		
delet:	call	left		; Rubout
		mvi 	c,20h		; Blank
		call	co_
		call	left
		ret	
;			
tabhor:	mvi 	a,htab
		mov 	b,a
		lda 	ioflg		; Paralleldruck
		push	psw   	; unterdruecken
		ani	0fh   	; (4 MSB ausblenden)
		sta	ioflg 	; kein Paralleldruck
tablop:	mvi	c,20h 	; Blank ausgeben
		call	co_	
		dcr	b	
		jnz	tablop	
		pop	psw   	; altes I/O-FLAG
		sta	ioflg 	; zurueckladen
		lhld	update	
		ret
;		
crlf_:	call	linfe_
; 
caret:	lhld	linz		; CR nach unten links!
		ret
;
tabver:	mvi	a,vtab
		mov	b,a
tbvlop:	call	linfe_
		dcr	b
		jnz	tbvlop
		ret	
;			
linfe_:	push	b
		push	d
		lxi 	b,vidram	; Ziel
		lxi 	d,vidend	; Ende
		lhld	lin2    	; Beginn
linlop:	in  	vidctl  	; Blanking abwarten
		rrc	
		jnc	linlop
		mov	a,m		; RAM auslesen und
		stax	b		; hochkopieren
		inx	b
		inx	h
		mov	a,d
		cmp	h		; Endabfrage
		jnz	linlop
		mov	a,e
		cmp	l
		jnz	linlop
		lhld	linz		; unterste Zeile loeschen
		call	clrli_
blk3:		in	vidctl	; Blanking abwarten
		rrc
		jnc	blk3
		mvi	m,20h		; hinter Bildfenster dunkel
		lhld	update	; Cursor-Pos. beibehalten!
		pop	d
		pop	b
		ret	
;		
clrli_:	push	b
		push	h
		lda 	hordsp
		mov 	e,a
		mvi 	d,0h
		dad 	d
		xchg			; D&E: Zeilenende (+1)
		pop 	h		; H&L: Zeilenanfang
clrcte:	mvi 	c,20h		; Blank
blk4:		in  	vidctl	; Blanking abwarten
		rrc	
		jnc 	blk4
		mov 	m,c
		inx 	h
		mov 	a,d
		cmp 	h		; Endabfrage
		jnz 	blk4
		mov 	a,e
		cmp 	l
		jnz 	blk4
		pop 	b
		ret	
;			
clrvi_:	.ds   0h
; 
clram:	push	b		; 
		lxi	h,vidram
		lxi	d,vidend
		inx	d
		jmp	clrcte
; 
shilok:	pop	h
		lxi	h,warm	; anschliessend Warmstart
		push	h     	; in den Stack
		lda	ioflg	
		xri	1h    	; LSB invertieren
		sta	ioflg	
		jmp	iofout	; I/O-FLAGs anzeigen
; 
;
;***************************************************
;
; Cursor-Steuerung im Monitor
;
;***************************************************
;
cursor:	pop	h		; cte-Adr. loeschen
		lxi	h,coend	; cte-Adresse
		push	h		; in den Stack
		mov	a,c
		cpi	80h		; CTL+H
		jz 	left
		cpi	82h		; CTL+J
		jz 	right
		cpi	88h		; CTL+DEL
		jz 	lleft
		cpi	89h		; CTL+U
		jz 	up
		cpi	8ah		; CTL+TAB
		jz 	rright
		cpi	8bh		; CTL+N
		jz 	down
		cpi	8fh		; CTL+Z
		jz 	home
		ret			; zu COEND
;
left:		push b
		lhld update
		dcx  h
		call ramin		; ausserhalb?
		jnc  lftend		; nein:JMP
		lxi  h,vidend
		dcx  h
lftend:	call crspo_
		pop  b
		ret
;
right:	push b
		lhld update
		inx  h
		call ramax
		jc   rigend
		lxi  h,vidram
rigend:	jmp  lftend
;
lleft:	mvi	a,htab	; CTL+DEL
		mov	b,a
llop:		call	left
		dcr	b
		jnz	llop
		ret	
;
up:		lhld	update
		lda	hordsp
		cma	
		mov	e,a
		mvi	d,0ffh
		inx	d 		; D&E=(-vidram)
		dad	d
		call	ramin
		jnc	upend
		xchg
		lxi	h,vidend
		mov	a,h
		ani	0fh		; 4 MSB ausblenden
		mov	h,a
		dad	d
upend:	jmp	crspo_
;
rright:	mvi	a,htab	; CTL+TAB
		mov	b,a
rlop:		call	right
		dcr	b
		jnz	rlop
		ret
;
down:		lhld	update
		lda	hordsp
		mov	e,a
		mvi	d,0h
		dad	d
		call	ramax
		jc	dowend
		xchg
		lxi	h,vidend
		mov	a,h
		ani	0fh		; 4 MSB unterdruecken
		cma	
		mov	h,a
		mov	a,l
		cma	
		mov	l,a
		inx	h		; H&L=(-vidram)
		dad	d
dowend:	jmp	upend
;
home:		lxi	h,vidram
crspo_:	shld	update
		mvi	a,0eh		; CRS-Reg
		out	vidreg
		mov	a,h
		ani	0fh		; 4 LSB maskieren
		out	vidat
		mvi	a,0fh		; CRS-Reg
		out	vidreg
		mov	a,l
		out	vidat
		ret	
;
crson_:	mvi	a,0ah		; CRS-Reg
		out	vidreg
		mvi	a,6ah		; CRS schmal
		out	vidat
		ret	
;	
crsof_:	mvi	a,0ah		; CRS-Reg
		out	vidreg
		mvi	a,20h		; CRS off
		out	vidat
		ret
;
ramin:	xchg			; D&E:Video-Pointer
		lxi	h,vidram
		mov	a,d 		; (A): Pointer (HI)
		cmp	h		; (H): LIN1 (HI)
		jnc	minend	; (A) nicht kl.(H):JMP
		jnz	minend	; (A) nicht gl.(H):JMP
		mov	a,e
		cmp	l
minend:	xchg
		ret
;
ramax:	xchg			; D&E:Video-Pointer
		lxi	h,vidend
		mov	a,d		; (A): Pointer (HI)
		cmp	h		; (H): LINUL (HI)
		jc 	maxend	; (A) kleiner (H):JMP
		jnz	maxend	; (A) nicht gl.(H):JMP
		mov	a,e
		cmp	l
maxend:	jmp	minend
;
;***************************************************
;
; Beeinflussung des I/O-FLAGs (CTL-Codes)
;
;***************************************************
;
contrl:	lxi	h,0ah		; 5 PUSHs ausgleichen
		dad  sp
		sphl			; SP unter RET-Adresse
		pop  h		; H&L: RET-Adresse
		mov  a,h
		ani  0f0h		; oberes Nibbel bleibt
		mov  h,a		; Warmstart-Adr.(upper)
		mvi  l,3h		; Warmstart-Adr.(lower)
		push h
		mov  a,c
		ani  07h		; 3 LSB maskieren
		rz			; CTL+0: RET
		mov  c,a		; CTL+1...7 (nicht EXT!)
		mvi  a,1h
ctlop:	rlc
		dcr  c
		jnz  ctlop
		mov  c,a		; Invers-Bit
		lda  ioflg
		xra  c		; Bit invertieren
iofout:	sta  ioflg
		push psw
		lxi  h,iotxt
		call strin_
		pop  psw
		call bytbin		; I/O-FLAGs ausgeben
		ret			; Warmstart im lfd.Programm
;		
iotxt:	.db	"I"
		.db	"/"
		.db	"O"
		.db	"-"
		.db	"F"
		.db	"L"
		.db	"A"
		.db	"G"
		.db	"s"
		.db	3ah
		.db	20h
		.db	0h
;
;***************************************************
;
; Ausgabe von Zeichenketten
;
;***************************************************
;
;      H&L: Pointer; Stringende: 00h
;
strin_:	xchg			; D&E:String-Pointer
		lhld	update	; H&L:RAM-Pointer
		nop	
strlop:	ldax	d
		ora 	a		; Ende?
		rz			; ja: RET
		mov 	c,a
		call	co_		; Ausgabe
		cpi 	0dh
		jnz 	strcte	; kein CR: JMP
		mvi 	c,0ah		; in STRING LF anhaengen
		call	co_
strcte:	inx 	d
		jmp 	strlop
;
undrl_:	xchg			; Inversdarstellung
		lhld	update
		ldax	d
		mov 	c,a
blk5:		in  	vidctl	; Blanking abwarten
		rrc	
		jnc 	blk5
		mov 	m,c		; erstes Zeichen ausgeben
		inx 	d
		inx 	h
undlop:	ldax	d
		ora 	a
		jz  	undend	; Stringende  
		ori 	80h		; Inversbit setzen
		mov 	c,a
blk6:		in  	vidctl	; Blanking abwarten
		rrc	
		jnc 	blk6
		mov 	m,c
		inx 	d
		inx 	h
		jmp 	undlop
undend:	in  	vidctl	; Blanking abwarten
		rrc 	
		jnc 	undend
		mov 	a,m
		ori 	80h		; letztes Zeichen auch invers
		mov 	c,a
blk7:		in  	vidctl	; Blanking abwarten
		rrc	
		jnc 	blk7
		mov 	m,c
		ret	
;
;      BREAK     verharrt bei gedrueckter Taste; bei
;                Eingabe des BREAK-Codes erfolgt
;                Sprung nach ((H&L)); kein Stack-
;                Ausgleich, kein CRSON!
;
brkcod	.equ	1bh		; CTL+C
;
break_:	push	psw
		push	h
		call	csts		; Taste aktiv?     
		jz  	brkend	; nein: JMP
		call	ci
		cpi 	brkcod	; Abbruch?
		jnz 	lokon		; nein: JMP
loklop:	call	csts 
		jnz 	loklop	; bei Loslassen
		pchl			; zum Warmstart
lokon:	call	csts
		jnz 	lokon		; auf Loslassen warten
lokof:	call	csts 
		jz  	lokof		; auf Tastendruck warten
lokout:	call	csts
		jnz 	lokout	; wieder Loslassen abwarten
;	
brkend:	pop 	h
		pop 	psw
		ret
;
;**************************************************
;                                                 *
;      RAM  Speicherverwaltung           13.11.85 *
;                                                 *
;**************************************************
;
;
;
;****************************************************
;
;      KILL  Zeichen durch anderes ersetzen
;
;****************************************************
;
;
; Ersetzt im Bereich der angegebenen Anfangs- und
; Endadressen ein Bitmuster durch ein anderes; Bei-
; spiel: Im Bereich 9000...AFFFh sollen alle vor-
; kommenden "FFh" eliminiert (durch 00h ersetzt)
; werden.
;
; Eingabe: "k 9000,AFFF,FF00"; Return
;
kill:		lhld	srcend
		xchg			; D&E: Endadresse
		lhld	dstbeg
		mov	b,l		; Ersatzzeichen
		mov	c,h		; zu ersetzendes Zeichen
		lhld	srcbeg	; H&L: Anfangsadresse
;
kilop:	mov	a,m
		cmp	c
		jnz	kicte		; nicht zu ersetzen: JMP
		mov	m,b		; Zeichen ersetzen
		mov	a,m		; Kontrollesen
		cmp	b
		jz 	kicte		; o.k.: JMP
		push	h
		push	d
		push	b
		mvi 	a,14h		; fehlgeschlagen:
		call	error		; Fehlermeldung
		pop 	b
		pop 	d
		pop 	h
kicte:	mov 	a,l
		cmp 	e		; Endabfrage
		jnz 	kict
		mov 	a,h
		cmp 	d
		jnz 	kict		; nicht am Ende: JMP
		lhld	srcbeg
		jmp 	copct2	; eine Zeile listen
kict:		inx 	h
		jmp 	kilop
;
;
;****************************************************
;
;      TXT  Ausgabe eines ASCII-Strings
;
;****************************************************
;
; Ausgabe eines Textes von der angegebenen Start-
; adresse an bis zum ebenfalls angegebenen End-
; zeichen; Beispiel: ASCII-String ab 9000h ausge-
; ben, bis das Endzeichen "00h" erreicht ist:
;
; Eingabe: "t 9000,00"; Return
;
txt:		call	crlf
		lhld	srcbeg
		lda 	srcend  
txtlop:	mov 	c,m
		cmp 	c
		jz  	txtend	; Endzeichen erreicht: JMP
		call	co
		inx 	h
		jmp 	txtlop
txtend:	xra 	a
		inr 	a		; FLAGs loeschen
		ret	
;
;
;***************************************************
;
;      GO  Anwenderprogramm starten
;
;***************************************************
;
go:		lhld	srcbeg	;
		xra	a
		cmp	h		; Nulleingabe?
		jnz	gocte		; nein: JMP
		cmp	l
		jnz	gocte
		lxi	h,rambeg	; RAM-Anfangsadresse laden
gocte:	shld	adres
		jmp	restor	; REGs laden, Start
;
;
;***************************************************
;
;      FILL      RAM mit Konstante fuellen
;
;***************************************************
;
fill_:	call	fill
		lhld	srcbeg
		jmp 	copct2
;
;
;***************************************************
;
;      COPY      Speicherbereich verschieben
;
;***************************************************
;
copy_:	call	copy
copcte:	lhld	dstbeg
		shld	srcbeg
copct2:	shld	srcend
		jmp 	list
;
;
;**************************************************
;
;      EXCHAN    Speicherbereich verschieben  und
;                dabei Adressen anpassen
;
;**************************************************
;
excha_:	call	exchan
		jmp	copcte
;
;
;***************************************************
;
;      LIST      HEX-Dump des Speicherinhaltes
;
;***************************************************
;
smlvid	.equ	28h		; HORDSP bei  Kleinformat
;
list:		lhld	srcbeg
		mov 	a,l
		ani 	0f0h		; abrunden        
		mov 	e,a
		mov 	d,h		; D&E: Anfangsadresse
		lhld	srcend
		mov 	a,l
		ori 	0fh		; aufrunden
		mov 	l,a		; H&L: Endadresse
		inx 	h		; wg. COMP
		call	crlf
		mvi 	a,0h
		sta 	hibuf2	; Zeilenzaehler
		xchg			; D&E:Ende;  H&L:Pointer
lilop:	call	adrot		; Adresse ausgeben
		xchg			; wg. ADROT-XCHG
		call	byt8		; 8 Bytes ausgeben
		jnz 	lict
lend:		xra 	a
		inr 	a		; Z-FLAG loeschen
		ret	
lict:		lda 	hordsp
		cpi 	smlvid
		jnz 	licte		; Grossformat: JMP
		jmp 	inrlin	; Linefeed bei Kleinformat
					; 
licte: 	call	byt8
		jz  	lend
inrlin:	call	crlf
		lda 	hibuf2
		inr 	a		; Zeilenzaehler
		sta 	hibuf2
		ani 	3fh		; mod. 64d?
		jnz 	lilop
		lda 	ioflg		; Paralleldruck?
		ani 	0f0h
		jz  	lilop
		mvi 	c,0ch		;Formfeed
		call	lo
		jmp 	lilop
;
byt8:		call	space
		call	byt4
		call	byt4
		call	comp
		ret
		
byt4:		call	space
		call	byt2
		call	byt2
		ret
;
byt2:		call	byt1
		call	byt1
		ret
;
byt1:		call	space
		mov 	a,m
		call	bytot
		push	d
		push	h
		lxi 	h,warm
		call	break
		pop 	h
		pop 	d
		inx 	h
		ret
;
spaces:	call	space
space: 	push	b
		mvi 	c,20h		; Blank
		call	co
		pop 	b
		ret	
;
;---------------------------------------------
;
comp:		mov	a,e
		cmp	l
		rnz	
		mov	a,d
		cmp	h
		ret	
;
;****************************************************
;
;      MEM       RAM-Inhalt inspiz./modifizieren
;
;****************************************************
;
mem:		lhld	srcend
		mov 	a,l
		ora 	h
		jnz 	fill_		; Par.2 nicht Null: JMP
;	
memode:	call	crson		; Memory-Mode
		call	crlf
		lda 	ioflg		; save IO-FLAG
		sta 	hibuf2
		ani 	0fh		; Paralleldruck
		sta 	ioflg		; ausschalten
		lhld	srcbeg
		xra 	a
		cmp 	h		; Nulleingabe?
		jnz 	memlop	; nein: JMP
		cmp 	l
		jnz 	memlop
		lxi 	h,rambeg
;
memlop:	shld	adres
memlp:	lhld	adres
		mov 	a,m
mlop:		sta 	data  	; Speicherinhalt auslesen
		call	adrot 	; Adresse ausgeben
		xchg	      	; wg. ADROT-XCHG
		call	spaces	
		mvi 	c,3eh 	; Daten-Pfeil (rechts)
		lxi 	d,0ah	
		lda 	modflg	
		ani 	1h	
		jnz 	memcte	; Daten-Mode: JMP
		mvi 	c,3ch 	; Adress-Pfeil (links)
		lxi 	d,3h	
memcte:	call	co	
		lhld	linz  	; Cursor auf Daten- oder
		dad 	d     	; Adressfeld
		push	h	
		call	spaces	
		lda 	data	
		push	psw	
		call	bytot 	; Daten ausgeben
		call	spaces	
		pop 	psw	
		call	bytbin	; Daten binaer ausgeben
		pop 	h	
		call	crspos	
			
		call	ci		; Tastatur abfragen
		cpi	1bh		; CTL+C?
		jz 	memend
		mov	b,a		; fuer Fehlerbeh. retten
		cpi	0dh		; RET?
		jz 	nxt
		cpi	20h		; Blank?
		jz 	nxt		; (=RET)
		cpi	2dh		; Minus?
		jz 	bak		
		cpi	3ch		; Adr?
		jz 	adr		
		cpi	3eh		; Dat?
		jz 	dat		
		
		lxi	d,0h
		call	hxtest	; HEX-Zeichen?
		jnz 	mprocs
		mov 	c,b
		call	co		; falsches Zeichen ausgeben 
		call	error
		call	crlf
		jmp 	memlp
;	
memend:	lda 	hibuf2	; restore IOFLG
		sta 	ioflg
		jmp 	warm
;	
mprocs:	mov 	b,a		; Verarbeitung
		lda 	modflg
		ani 	1h
		lhld	adres
		jz  	sftadr	; Adress-Mode: JMP
sftdat:	lda	data		; Daten nachschieben
		ani	0fh
		rlc	
		rlc	
		rlc	
		rlc	
		ora	b
		mvi	c,0dh
		call	co
		jmp	mlop
sftadr:	mov	a,h		; Adresse nachschieben
		ani	0fh
		rlc	
		rlc	
		rlc	
		rlc	
		mov	h,a
;	
		mov	a,l
		rlc	
		rlc	
		rlc	
		rlc	
		mov	l,a
		ani	0fh
		ora	h
		mov	h,a
;	
		mov	a,l
		ani	0f0h
		ora	b
		mov	l,a
		mvi	c,0dh
		call	co
		jmp	memlop
;
nxt:		lhld	adres		; Adresse erhoehen
		lda	data
		mov	m,a
		mov	b,m
		cmp	b
		jz 	nxtcte	; Einschreiben o.k.:JMP
		mvi	a,14h
		call	error		; Einschreiben fehlgeschl.
		jmp	memlp		; Adresse bleibt erhalten
nxtcte:	inx	h
		call	prtcrl
		jmp  memlop
;
bak:		call	prtcrl	; Adresse erniedrigen
		call	prtcrl
		lhld	adres
		dcx 	h
		jmp 	memlop
;
adr:		lda	modflg	; Adress-Mode waehlen
		ani	0feh
		ori	2h
adrcte:	sta	modflg
		mvi	c,0dh
		call	co
		jmp	memlp
;
dat:		lda	modflg	; Daten-Mode waehlen
		ani	0fdh
		ori	1h
		jmp	adrcte
;
prtcrl:	lda  hibuf2		; CRLF mit evtl.Paralleldruck
		sta  ioflg
		call crlf
		lda  ioflg
		sta  hibuf2
		ani  0fh
		sta  ioflg
		ret
;
;
;***************************************************
;
;      COMPAR  Speicherbereiche vergleichen
;
;***************************************************
;
; Listet die Abweichungen auf (links=SRCBEG,
; rechts:DSTBEG)
;
compa_:	call	crlf
cmplop:	lhld	dstbeg
		xchg			; D&E:Zielbereich
		lhld	srcbeg	; H&L:Quellbereich
		ldax	d
		cmp 	m		; Vergleich
		jz  	ready		; gleich: Endabfrage
			
		push	h
		call	adrot		; Quelladresse ausgeben
		mvi 	c,3dh		;  "="
		call	co
		pop 	h
		mov 	a,m
		call	bytot		; Inhalt Quell-Speicher
		call	spaces
		lhld	dstbeg
		push	h
		call	adrot		; Zieladresse ausgeben
		mvi 	c,3ah		; ":"
		call	co
		pop 	h
		mov 	a,m
		call	bytot		; Inhalt Ziel-Speicher
		call	crlf
		lxi 	h,warm
		call	break		; Abbruch?
; 	
ready:	lhld	srcend
		xchg	
		lhld	srcbeg
		mov 	a,l
		cmp 	e
		jnz 	rdyend
		mov 	a,h
		cmp 	d
		jnz 	rdyend
		xra 	a		; Endadresse erreicht
		inr 	a		; FLAGs loeschen
		ret	
rdyend:	inx 	h
		shld	srcbeg
		lhld	dstbeg
		inx 	h
		shld	dstbeg
		jmp 	cmplop
;
;*************************************************
;                                                *
; PERIF  Ansteuerung peripherer Baugr.  13.11.85 *
;                                                *
;*************************************************
;
;
;****************************************************
;
; PRM  Aufruf der EPROM-Programmier-Software
;
;****************************************************
;
; Unterscheidet 87er- und 89er-Prommer-Har.dware
; durch Abfrage der Adresse 2000h; in der 89er-Hard-
; ware steht dort (nach Umschaltung auf Hilfsbank 3)
; ein NOP (=00h), andernfalls ein JMP (=C3h) bzw.FFh.
;
prm:		mvi	a,0c0h
		out	fdctrl	; Hilfsbank 3 einschalten
		lda	2001h 	; Testadresse auslesen
		cpi	0h		; 00h nur bei 89er-Har.dware
		jz 	2000h 	; dann dorthin
		jmp	promsw	; 87er-Software hinter Editor
;
;
;****************************************************
;
;      Bildschirmabzug
;
;****************************************************
;
hrdcpy:	lxi	h,ioflg	; I/O-FLAG einstellen
		lda	srcbeg	; N0,4,5,6,7 erlaubt!
		mvi	m,10h		; =LO4
		cpi	4h		; Eingabe N4?
		jz 	print		; ja: JMP
		mvi	m,20h		; =LO5
		cpi	5h		; Eingabe N5?
		jz 	print		; ja: JMP
		mvi	m,40h		; =LO6
		cpi	6h		; Eingabe N6?
		jz 	print		; ja: JMP
		mvi	m,80h		; Eingabe 0 oder 7: LO7
;
print: 	lhld	linz
		dcx 	h
		xchg			; D&E: Bufferende
		lxi 	h,vidram
prloop:	mov 	c,m
		call	lo
		mov 	a,l
		cmp 	e
		jnz 	prtcte
		mov 	a,h
		cmp 	d
		jnz 	prtcte	; kein Ende: weiter
		xra 	a
		inr 	a		; FLAGs loeschen
		ret	
prtcte:	inx 	h
		jmp 	prloop
;
;
;****************************************************
;
; V24OT/V24IN  Verwaltung der V.24-Schnittstelle
;
;****************************************************
;
; Einsprung erfolgt aus Men-Tabelle
; SRCBEG = Startadresse RAM; SRCEND = Endadresse RAM fr Datenbuffer
;
vvot:		lxi 	h,v24ot_	; V.24-Ausgabe
		jmp 	vvcte
vvin:		lxi 	h,v24in_	; V.24-Eingabe
;	
vvcte:	shld	hbuf2		;  In Buffer JMP v24in-/v24ot- schreiben
		mvi 	a,0c3h	; "JMP"
		sta 	hibuf2
		lxi 	h,inosub	; RET-Adresse sichern (Rcksprung nach inosub)
		push	h		; in den Stack
		lhld	v24ini	; =CALL V24INI Timer mit Baudraten laden
		pchl	
;	
inosub:	lhld	srcbeg	;  Schleife bis SRCBEG = SRCEND
		mov 	c,m
		push	h
		call	hibuf2	; I/O-Subroutine aufrufen v24in-/v24ot-
		pop 	d
		lhld	srcend
		xchg	
		mov 	a,l
		cmp 	e
		jnz 	inrhl
		mov 	a,h
		cmp 	d
		jnz 	inrhl		; I/O fortsetzen
inoend:	call	loinit	; wieder Drucker einstellen
		xra 	a
		inr 	a		; FLAGs loeschen
		ret	
;	
inrhl:	inx 	h		; Pointer erhoehen
		shld	srcbeg
		jmp 	inosub	;  Schleife bis alle Daten empfangen/gesendet wurden
; 	
v24ot_:	lxi 	d,3h		; Sprungziele ermitteln Adr=v24ini + 3
		jmp 	v24
v24in_:	lxi 	d,6h		;  Adr=v24ini + 6
v24:		lhld	v24ini	;  Bezugsadresse aus Monitor rot laden
		dad 	d
		pchl			;  Sprung zur eigendlichen V24-IO-Routine
;
;
;*****************************************************
;
; EZU  Ansteuerung der Echtzeit-Uhr
;
;*****************************************************
;
;  Stellen der Uhr: Kommando "u", gefolgt von
;                   Stunden/Minuten,Wochentag-   
;                   Code (1=Montag)/Kalendertag,  
;                   Monat/Jahr; Return.
;  Beispiel: Fr,08.11.85;15:47:00h wird eingegeben
;            als 1547 (15:47h; Sekunden immer 00),
;            508 (Freitag der 8.), 1185 (Nov.85).
;
;  Lesen der Uhr: Kommando "u" ohne weitere Parameter
;                 
;  Anschliessend wird die Uhrzeit automatisch rechts
;  oben in den Bildschirm eingeblendet; dazu wird
;  der Interrupt RST6.5 (beim 8085) bzw. RSTB (beim
;  NSC800 benutzt.
;
ezu		di
		lhld	srcbeg
		mov 	a,h
		ora 	l		; Nullabfrage
		jz  	rdezu		; Null: Uhr lesen
; 	
wrezu:	lhld	srcend	; Wo/Kal-Tag
		mov 	b,h
		mov 	c,l
		lhld	dstbeg	; Monat/Jahr
		xchg	
		lhld	srcbeg
		call	writur
;	
rdezu:	lxi 	h,int65	; RST6.5-ISR vorbereiten
		mvi 	m,0c3h	; "JMP" 
		inx 	h
		lxi 	d,isr65
		mov 	m,e		; Zieladresse ins RAM
		inx 	h
		mov 	m,d
		mvi 	a,1dh		; RST6.5 freigeben
		sim	
		nop			; wg.NSC800-JR-Befehl
		mvi 	a,4h		; RSTB freigeben
		out 	0bbh
			
		mvi 	c,0ah		; Linefeed
		call	co
		call	time
		lxi 	h,ascbuf
		call	string	; Uhrzeit/Datum ausgeben
		inr 	c		; ZERO-FLAG loeschen
		nop			; ./. EI
;
; RST6.5-Interrupt-Service-Routine (ca.3 ms b. 6 MHz)
;
isr65:	push	psw
		push	h
		push	d
		push	b
			
		lhld	second
		inx 	h		; Sekundenzaehler erhoehen
		shld	second	; (max. 18:12:16 h)
			
		lhld	lin2		; Bildschirmeinblendung
		lxi 	d,0ffebh	; oben rechts vorbereiten
		dad 	d		; H&L: Bildposition
		push	h
		call	time		; ASCII-String erzeugen
		pop 	h
		lxi 	d,ascbuf
;	
wrtlop:	ldax	d		; ASCII-Code auslesen
		ora 	a		; Endabfrage
		jz  	isrend	; Stringende: JMP
		mov 	c,a
wrtlp: 	in  	vidctl	; Blanking abwarten
		rrc			; (nicht CO wg.Par.Druck)
		jnc 	wrtlp
		mov 	m,c		; Code ausgeben
		inx 	d
		inx 	h
		jmp 	wrtlop
;	
isrend:	pop 	b
		pop 	d
		pop 	h
		pop 	psw
		ei	
		ret	
;
;
;****************************************************
;
; BYTE  Einzel-Byte-Betrieb
;
;****************************************************
;
;  Setzt das Einzelschritt-Modul voraus (bzw.89er-CPU
;  mit LED-Anzeige)
;
byte:		lhld	srcbeg	;Startadresse
		mvi	a,40h
		out	stpff		;STEP-Flipflop
		pchl
;
;
;****************************************************
;
; BREAK  Breakpoint-Betrieb im RAM
;
;****************************************************
;
; Der OpCode an der naechstfolgenden (bzw. an der
; Break-)Adresse wird durch einen Software-Inter-
; rupt (RS2=D7h) ersetzt; deshalb muessen die im
; BREAK- oder STEP-Mode durchlaufenen Programm-
; teile im RAM stehen.
;
; Im Anwenderprogramm duerfen nicht die Befehle
; RST1 und RST2 verwendet werden (fuer den Monitor
; reserviert); der User-Stack-Pointer darf, wenn
; er modifiziert wird, nur unterhalb 2E00h liegen.
;
posa		.equ	8h		; Pos. von REG-A-Daten
leng		.equ	27h		; String ges. 39d lang
len		.equ	1fh		; Reg-String 31d lang
;
brkpt:	call	brkini	; Breakpoints vorbereiten
		lhld	srcend	; Break-Adresse
		shld	adres 	; uebergeben und retten
bresto:	shld	brkpt1	; BREAK-RESTOR-Ersatz
		mov 	a,m   	; OpCode holen
		sta 	opcod1	; und retten
		mvi 	m,0d7h	; "RST2" einsetzen 
		lda 	regf  	; BREAK-RESTOR-Ersatz
		mov 	c,a   	; REG C: CPU-FLAGs
		push	b	
		pop 	psw   	; CPU-FLAGs setzen
		lhld	regs  	; User-Stack-Pointer
		mov 	a,l   	; sortieren
		mov 	l,h   	; (H&L vertauschen)
		mov 	h,a	
		sphl	      	; in CPU-Stack-Pointer
		lxi 	h,regb	; Pointer auf REG-Buffer
		mov 	b,m   	; REG B laden
		inx 	h	
		mov 	c,m   	; REG C laden
		inx 	h	
		mov 	d,m   	; REG D laden
		inx 	h	
		mov 	e,m   	; REG E laden
		lhld	srcbeg	; Startadresse holen
		push	h     	; und im Stack ablegen
		lhld	regh  	; Daten fuer H&L holen
		mov 	a,l   	; und sortieren
		mov 	l,h   	; ((H&L vertauschen)
		mov 	h,a	
		lda 	rega  	; REG A laden
		ret 	      	; Start bis zum Breakpoint
;
;
;******************************************************
;
; STEP  Einzelschritt-Betrieb im RAM
;
;******************************************************
;
; Es sind dieselben Ran.dbedingungen zu beachten wie
; beim Breakpoint-Betrieb.
;
step:		call	brkini
		lhld	srcbeg
		shld	adres
		jmp 	bresto	; D7h einsetzen und Start
;	
brkcte:	call	crlflo	; CRLF nur an Printer
		call	crlflo
		lda 	ioflg		; Kopfzeile ausdrucken?
		ani 	0f0h
		jz  	inloop	; kein Paralleldruck: JMP
		lxi 	h,hedlin	; Indexleiste
prtlop:	mov 	a,m
		ora 	a
		jz  	prtend	; Ende: JMP 
		mov 	c,a
		call	lo		; ausdrucken
		inx 	h
		jmp 	prtlop
prtend:	call	crlflo
		mvi 	c,2dh		;  "-"
		mvi 	b,leng
prtlp:	call	lo		; Unterstreichung
		dcr 	b
		jnz 	prtlp
		jmp 	inloop
;
;
; BRKINI  Breakpoints vorbereiten
;
brkini:	lxi	h,brkcte
		shld	hbuf2 	; CTE-Adresse (Warmstart)
		mvi	a,0c3h	; "JMP"
		sta	hibuf2	; CTE-Adresse fuer HIBUF
		lda	modflg	; Mode-FLAG holen
		ani	0f7h  	; FCT-FLAG loeschen
		ori	80h   	; STP-Bit setzen
		sta	modflg	
		xra	a	
		sta	regf  	; REG F loeschen
		lxi	h,0h  	; Breakpoint-Adr. loeschen
		shld	brkpt1	
		shld	brkpt2	
		ret
;
;
;****************************************************
;
; INLOP  Register-Inhalte anzeigen und modifizieren
;
;****************************************************
;
; Bewegung des Cursors nach links und rechts ueber
; die Pfeiltaste neben "TAB", die Cursor-Steuer-
; tasten sowie CTL+H (links) bzw. CTL+J (rechts)
;
inloop_:	lxi 	h,inlop
		shld	hbuf2
		call	crlf2
inlop_:	lhld	linz
		shld	update
		call	clrlin
		lhld	adres
		push	h
		call	adrot		; RAM-Adresse anzeigen
		mvi 	c,20h		; Blank
		call	co  
		pop 	h
		mov 	a,m
		call	bytot		; RAM-Inhalt anzeigen
		call	regsot	; REG-Inhalte anzeigen
; 	
		call	crlf
		lda 	ioflg		; Paralleldruck
		push	psw		; unterdruecken
		ani 	0fh
		sta 	ioflg
		lxi 	h,hedlin	; Indexzeile ausgeben
		call	undrln
		pop 	psw
		sta 	ioflg		; alter Zustand
		lhld	liny		; vorletzte Zeile
		lxi 	d,posa
		dad 	d		; auf REG A posit.
		call	crspos
		call	crson
		lxi 	sp,stack	; Monitor-Stack!
;	
inlp:		call	ci
		cpi 	0dh		; Ret?
		jz  	nxt_
		cpi 	20h		; Blank?
		jz  	nxt_
		cpi 	3ch		; Pfeil links?
		jz  	left_
		cpi 	80h		; Cursor links?
		jz  	left_
		cpi 	3eh		; Pfeil rechts?
		jz  	right
		cpi 	82h		; Cursor rechts?
		jz  	right
		cpi 	1bh		; CTL+C?
		jz  	warm		; Monitor-Ruecksprung
			
		call	hxtest	; Eingabe verarbeiten
		jnz 	incte_	; HEX-Input: JMP
		out 	belout	; Eingabe ignorieren
		jmp 	inlp
incte_:	ori 	30h		; ASCII-Umsetzung
		cpi 	3ah		; "9"+1
		jc  	cte
		adi 	7h		; Buchstabe
cte:		mov 	c,a
		lhld	update
blklop:	in  	vidctl	; Blanking abwarten
		rrc	
		jnc 	blklop
		mov 	m,c		; Bildschirm-Ausgabe
movcrs:	inx 	h
		call	crspos
		push	h
		lhld	liny		; Endabfrage
		lxi 	d,leng
		dad 	d		; H&L:Zeilenende
		pop 	d		; D&E:Cursor-Pos.
		xchg	
		mov 	a,l
		cmp 	e
		jnz 	inct_
		mov 	a,h
		cmp 	d
		jnz 	inct_		; nicht am rechten Rand: JMP
		lhld	liny		; Cursor auf REG A
		lxi 	d,posa
		dad 	d
		call	crspos
		jmp 	inlp
inct_:	in  	vidctl	; Blanking abwarten
		rrc	
		jnc 	inct_
		mov 	a,m
		cpi 	20h		; auf Blank?
		jnz 	inlp
		jmp 	right		; Cursor weiter rechts
;	
nxt_:		call	input		; Daten vom Bildschirm einlesen
		jmp 	restor	; REGs laden und Start
;	
left_:	lhld	update
		dcx 	h
		call	crspos
		push	h
		lhld	liny		; links von REG A?
		lxi 	d,posa
		dcx 	d
		dad 	d
		pop 	d
		mov 	a,l
		cmp 	e
		jnz 	lftcte 
		mov 	a,h
		cmp 	d
		jnz 	lftcte	; nein: weiter
		lxi 	d,len
		dad 	d
		call	crspos
		jmp 	inlp
lftcte:	xchg	
lftlop:	in  	vidctl	; Blanking abwarten
		rrc	
		jnc 	lftlop
		mov 	a,m
		cpi 	20h		; auf Blank?
		jnz 	inlp
		jmp 	left
;	
right_:	lhld	update
		jmp 	movcrs
;	
crlf2:	mvi 	c,0ah
		call	co		; Linefeed
		mvi 	c,0dh
		call	co		; Carriage Return
		ret	
; 	
crlflo:	mvi 	c,0ah		; CRLF nur fuer Printer
		call	lo
		mvi 	c,0dh
		call	lo
		ret	
;
;
; INPUT  Daten vom Bildschirm einlesen
;
input:	lhld	liny
		lxi 	d,posa	; bei REG A anfangen
		dad 	d
		call	regin
		sta 	rega		; REG A laden
		inx 	h
		call	regin
		sta 	regb		; REG B laden
		call	regin
		sta 	regc		; REG C laden
		inx 	h
		call	regin
		sta 	regd		; REG D laden
		call	regin
		sta 	rege		; REG E laden
		inx 	h
		call	regin
		sta 	regh		; REG H laden
		call	regin
		sta 	regl		; REG L laden
		inx 	h
		call	regin
		sta 	regs		; SP (upper) laden
		call	regin
		sta 	regp		; SP (lower) laden
		inx 	h
		call	flgin		; FLAGs einlesen
		sta 	regf
		ret	
;
regin:	in   vidctl		; Blanking abwarten
		rrc
		jnc  regin
		mov  a,m		; upper Nibble holen
		call hxtest
		rlc
		rlc
		rlc
		rlc
		mov  c,a
		inx  h
;
reglop:	in   vidctl		; Blanking abwarten
		rrc
		jnc  reglop
		mov  a,m		; lower Nibble holen
		call hxtest
		ora  c		; Byte zusammensetzen
		inx  h
		ret
;
flgin:	lxi  b,8h		; B=00, C=08
flglop:	in   vidctl		; Blanking abwarten
		rrc
		jnc  flglop
		mov  a,m		; Bit holen
		ani  1h		; LSB maskieren
		push psw
		rrc
		pop  psw
		jnz  flgcte
		mvi  a,0h		; Null nachschieben
flgcte:	ora  b		; Bit ins SR
		rlc
		mov  b,a
		inx  h
		dcr  c
		jnz  flglop
		mov  a,b
		rrc
		ret
;
;
hedlin:	.db	"A"
		.db	"d"
		.db	"r"
		.db	2eh
		.db	"I"
		.db	"n"
		.db	"h"
		.db	2eh
		.db	"A"
		.db	20h
		.db	20h
		.db	"B"
		.db	"&"
		.db	"C"
		.db	20h
		.db	20h
		.db	"D"
		.db	"&"
		.db	"E"
		.db	20h
		.db	20h
		.db	"H"
		.db	"&"
		.db	"L"
		.db	20h
		.db	20h
		.db	20h
		.db	"S"
		.db	"P"
		.db	20h
		.db	20h
		.db	"S"		; SIGN
		.db	"Z"		; ZERO
		.db	"x"		; Overflow (16 Bit)
		.db	"c"		; Aux. CARRY
		.db	"x"		; 0
		.db	"P"		; PARITY
		.db	"x"		; Overflow (bin.)
		.db	"C"		; CARRY
		.db	0h
;
;
;****************************************************
;
;      HELP  Menue-Anweisungen listen
;
;****************************************************
;
help:		lxi	h,hlptxt
		call	string
		inr	a		; FLAGs loeschen
		ret
hlptxt:	.ds	0h
;
		.db	0Dh		;  "CR"
		.db	"@:user"
		.db	0Dh			
		.db	"a*Assembler"
		.db	0Dh			
			
		.db	"b*BASIC"
		.db	0Dh			
			
		.db	"c:copy"
		.db	0Dh			
			
		.db	"d*Disassembler"
		.db	0Dh			
			
		.db	"e*Editor"
		.db	0Dh		
			
		.db	"f*Floppy"
		.db	0Dh		
			
		.db	"g:go"
		.db	0Dh		
			
		.db	"h:help"
		.db	0Dh		
			
		.db	"i:casin"
		.db	0Dh		
			
		.db	"j:jump"	
		.db	0Dh		
			
		.db	"k:kill"
		.db	0Dh		
			
		.db	"l:list"
		.db	0Dh		
			
		.db	"m:mem/fill"
		.db	0Dh		
			
		.db	"n:hardcopy"
		.db	0Dh		
			
		.db	"o:casot"
		.db	0Dh		
			
		.db	"p*Prommer"
		.db	0Dh		
			
		.db	"q:sample"
		.db	0Dh		
			
		.db	"r:revise"
		.db	0Dh		
			
		.db	"s:step"
		.db	0Dh		
			
		.db	"t:text"
		.db	0Dh		
			
		.db	"u:uhr"
		.db	0Dh		
			
		.db	"v:v24out"
		.db	0Dh		
			
		.db	"w:v24in"
		.db	0Dh		
			
		.db	"x:exchange"
		.db	0Dh		
			
		.db	"y:byte"
		.db	0Dh		
			
		.db	"z:break"
		.db	0Dh		
			
		.db	"*"
		.db	0Dh		
		.db	00h		; String Ende
;
;------------------------------------------------------
; RAM-Belegung
; Stack, Buffer etc.
;------------------------------------------------------
;
		.org	buff_th
thbuf:	.ds	17h
;
		.org  buff_fkt	; FCT-Sprungziele (zuerst unter
fctbuf:	.ds   0h        	; dann obere Adresshaelfte ablege
fct7:		.ds   02h
fct6:		.ds   02h
fct5:		.ds   02h
fct4:		.ds   02h
fct3:		.ds   02h
fct2:		.ds   02h
fct1:		.ds   02h
fct9_:	.ds   02h		; Extension
fct7_:	.ds   02h		; dito
fctdel:	.ds   02h
fct_:		.ds   02h
fctsz:	.ds   02h
fct0:		.ds   02h
fct9:		.ds   02h
fct8:		.ds   02h
fct8_:	.ds   02h		; dito
;
		.org	buff_vid	; Video-Parameter
hordsp:	.ds	1h		; Zeichen/Zeile (-1)
verdsp:	.ds	1h		; Zeilen/Seite
lin2:		.ds	2h		; 2.Zeile
liny:		.ds	2h		; vorletzte Zeile
linz:		.ds	2h		; letzte Zeile
update:	.ds	2h		; Cursor-Position
;
		.org	buff_ln	; LINFED-Buffer
linbuf:	.ds	50h		; 80 Zeichen
;
		.org	buff_sta	;  Monitor-Stack
stack  	.ds	40h		; 40 Level tief
;
		.org	buff_ass    ; Assembler-Bereich
assbuf:	.ds	64h
;
;		Anwender-Stack
usrstk	.equ	2f70h		; 6 Level tief
;
		.org	buff_fdc	; FDC-Bereich
fdcbuf:	.ds	1ch
track0:	.equ	2f75h		; Spur-Regs.
track2:	.equ	2f77h

;
		.org	buff_irq	; Interrupt-Sprungverteiler
intrap:	.ds	3h
int3:		.ds	3h
int4:		.ds	3h
int5:		.ds	3h
int55: 	.ds	3h
int6:		.ds	3h
int65: 	.ds	3h
int7:		.ds	3h
int75: 	.ds	3h
;
;      Hilfszellen fuer den Monitor
;
rega:		.ds	1h		; REG A
regb:		.ds	1h		; REG B
regc:		.ds	1h		; REG C
regd:		.ds	1h		; REG D
rege:		.ds	1h		; REG E
regf:		.ds	1h		; FLAGs
regh:		.ds	1h		; REG H
regl:		.ds	1h		; REG L
regs:		.ds	1h		; Stack-Pointer (HI)
regp:		.ds	1h		; Stack-Pointer (LO)
		.ds	1h		; frei
;
dspbuf:	.ds	1h		; LED-Zeile
dig1:		.ds	1h		; Digit 1
dig2:		.ds	1h		; Digit 2
dig3:		.ds	1h		; Digit 3
dig4:		.ds	1h		; Digit 4
dig5:		.ds	1h		; Digit 5
dig6:		.ds	1h		; Digit 6
dig7:		.ds	1h		; Digit 7
;		
data:		.ds	1h		; Daten
adres:	.ds	1h		; Adresse
adrhi: 	.ds	1h		
modflg:	.ds	1h		; Mode-FLAG
fctbf: 	.ds	1h		; FCT-Nr.
srcbeg:	.ds	2h		; Par.1
srcend:	.ds	2h		; Par.2
dstbeg:	.ds	2h		; Par.3
dstend:	.ds	2h		; Par.4
regnr: 	.ds	1h		; REG-Nr.
ioflg: 	.ds	1h		; I/O-Param.
nxtbuf:	.ds	1h		; FCT-Eingaben
maxbuf:	.ds	1h		; max. Anz.
keybyt:	.ds	1h		; KEY true./.invers
ciflag:	.ds	1h		; FLAG f. CI-Routine
hibuf: 	.ds	2h		; I/O-Hilfsroutine
hbuf		.ds	1h		
;
hibuf2:	.ds	1h		; CTE-Sprungziel
hbuf2:	.ds	2h		
brkpt1:	.ds	2h		; BRK-Adresse #1
opcod1:	.ds	1h		; OpCode-Buf. #1
brkpt2:	.ds	2h		; BRK-Adresse #2
opcod2:	.ds	2h		; OpCode-Buf. #2
;			
second:	.ds	2h		; Sekundenzaehler
;
		.org	buff_uhr	; BCD-Uhrzeit-Buffer
uhrbuf	.ds	1h		; Sek-Einer
		.ds	1h		; Sek-Zehner
		.ds	1h		; Min-Einer
		.ds	1h		; Min-Zehner
		.ds	1h		; Std-Einer
		.ds	1h		; Std-Zehner
		.ds	1h		; Tag-Einer
		.ds	1h		; Tag-Zehner
		.ds	1h		; Wochentag 1-7 Montag-Sonntag
		.ds	1h		; Dummy (nicht benutzt
		.ds	1h		; Monat-Einer
		.ds	1h		; Monat-Zehner
		.ds	1h		; Jahr-Einer
		.ds	1h		; Jahr-Zehner
		
		.org	buff_asc	; ASCII-Uhrzeit-Buf.
ascbuf	.ds	0eh		; Aufbau wie oben
;
;---------------------------------------------------
;
;
.end

