;********************************************************************
; Monitor (rot) berichtigte Orginalfassung von 1985
;
; mrt55-18.1.asm	Monitor rot V55 Stand 2018
; 
; Profi-Monitor V5.5	
; (C)hms85 Reinhard Gler  1985
; Neu aufgebaut W.Rmer 2015-2018
;
; Editor:		notepad++ auf TAB=6
; Assembler:	TASM (C)Thomas N.Anderson
;	
; Stand:
;	28:01:2018 - Berichtigungen/Ergaenzungen
;	24.01.2018 - Quellcode ueberarbeitet
;			 TAB auf 6 gesetzt, damit Listfile = Quellcode ist
;	
;	06.06.2017 - Tabelle Baudrate, V24 Modus
;			 und Betriebsart auf Asynchron berichtigt	
;	05.06.2017 - Anpassungen fuer TASM	(.org,.equ,.db etc.)
;	18.05.2017 - Fortsetzung		Entwanzt/ASM OK
;	17.05.2017 - Fortsetzung		Einzelschrittmodul
;	16.05.2017 - Fortsetzung		Hex-Tastatur
;	15.05.2017 - Fortsetzung		Perepheriegeraete
;	14.05.2017 - Kommentare/Label ergnzt	Deklarationsteil
;	20.01.2015 - Analyse V24-Schnittstelle
; 	05.01.2015 - symbolische Adressen
;
; Anmerkung: Berichtigung aus HAT1 eingepflegt
;
; Quellcode mit notepad++ bearbeitet (Tab=6)
; Anpassungen fuer TASM (Thomas N.Anderson)
; 	Instruktions-/Parameterteil einheitlich in Kleinbuchstaben 
;     weil TASM zwischen klein/gross unterscheidet
;                    
;--------------------------------------------------------------------
; Constante/Definitionen
;
;
rst5on	.equ	1eh		;8085
rstcon	.equ  02h		;NSC800
rst6on	.equ  1dh
rstbon	.equ  04h
rst7on	.equ  1bh
rstaon	.equ  08h
rstoff	.equ  17h
nscoff	.equ  00h
;
;---------------------------------------------------------
;
; IO-Adressen
;
; 87er V24-Schnittstellenkarte
;
; ACIA 6850
acia_ctl	.equ	0e9h		;  Control/Statusregister
acia_dat	.equ  0f9h		;  Daten lesen/schreiben
; Modus-Schalter
cs_mode	.equ  0a9h		; Modus (Eingangswahlschalter)
cs_bell	.equ  0b9h		; Bell  (Schnarre)
; Timer
timer_ctl	.equ  0d9h		; Controlregister
timer_0	.equ  89h		; Zaehler 0
timer_1	.equ  99h		; Zaehler 1 * nicht benutzt
timer_2	.equ  0c9h		; Zaehler 2 * nicht benutzt
;
; FDC-Karte
;
fdc_sel	.equ	40h		; Floppy Selekt-Byte
fdc_ctl	.equ	48h		; WD1720 Controlregister
fdc_sp	.equ	49h		; Spurregister
fdc_sek	.equ	4ah		; Sektorregister
fdc_dat	.equ	4bh		; Datenregister
;
; Parellelschnittstelle
; 8255
par_ctl	.equ	0d8h		; Control/Statusregister
par_pa	.equ	0a8h		; Port A
par_pb	.equ	0b8h		; Port B
par_pc	.equ	0c8h		; Port C
;
; Echtzeituhr
; 5832
uhr_ctl	.equ	19h		; Controlregister
uhr_dat	.equ	18h		; Datenregister
;
; Thermodrucker
;
therm_d	.equ	38h		; Daten 
therm_ctl	.equ	39h		; Controlregister
;
; EPROM-Brenner
;
prom_ctl	.equ	0bh		; Controlregister
prom_adrl	.equ	09h		; Adressen untere Haelfte
prom_adrh	.equ	0ah		;          obere Haelfte
prom_dat	.equ	08h		; Daten
;
; Einzelschrittmodul
;
ssm_ff	.equ	1ah		; Singlestep-Modul FlipFlop
;
; Videokarte
;
vid_dspen	.equ 	2ah		; Format/Blanking 
					; (Umschaltung Zeichensatz/ Abfrage DiSPlayENable
					;
lin		.equ	50h		; 80d Zeichen pro zeile
lininv	.equ	0ffb0h	; - 80d + 1
;
; Hex-display
;
led_zeil	.equ	00h		; LED-Zeile
hex_d1	.equ	10h		; DIGIT 1
hex_d2	.equ	20h		; DIGIT-2
hex_d3	.equ	30h		; DIGIT-3
hex_d4	.equ	40h		; DIGIT-4
hex_d5	.equ	50h		; DIGIT-5
hex_d6	.equ	60h		; DIGIT-6
hex_d7	.equ	70h		; DIGIT-7
outpor	.equ	80h		; DIGIT-7 (+10h)
;
; Hex-tastatur
;
comand	.equ	0bch		; CMD-Keys
hexhi		.equ	0dch		; Hex 8..F
hexlo		.equ	0ech		; Hex 0..7
;
nxtnr		.equ	0h		; NXT-Spalte
fctnr		.equ	2h		; FCT-Spalte
adrnr		.equ	4h		; ADR-Spalte
runr		.equ	6h		; RUN-Spalte
diodnr	.equ	7h		; Ge-Diode (zur Unterscheidung 
;					; HEX-/ASCII-Tastatur)
;
; ASCII-Tastatur			Adressbereich 0ch bis fch
;
ascii		.equ	0fch		; oberste Adresse
;					; 
;
;--------------------------------------------------------------------
;
; bis  2DFFh     :Anwender-RAM-Bereich
; ab 2E00h Buffer-Adressen
;
; Adressen hier zusammengefasst
;
buff_poi:	.equ	2e00h		; Buffer f.Thermo-Drucker
buff_num:	.equ	2e02h		; Zeichenzaehler 
buff_txt:	.equ	2e03h		; Textbuffer 
buff_fkt:	.equ	2e20h		; Fct-Tasten
buff_vid:	.equ	2e4eh			; Video
buff_ln:	.equ	2e60h		; Linebuffer 80 Zeichen
buff_mon:	.equ	2eb0h		; Monitor
buff_sta:	.equ	2f00h		; Stack
buff_ass:	.equ	2f00h		; Assembler
buff_usr:	.equ	2f70h		; Userstack
buff_fdc:	.equ	2f70h		; FDC-Routinen
buff_irq:	.equ	2f8ch		; Interuptverteiler
buff_uhr:	.equ	2fdch		; Uhr-BCD
;		
uhrseke:	.equ	2fdch		; Sekunden 	Einer
uhrsekz:	.equ	2fddh		; 		Zehner
uhrmine:	.equ	2fdeh		; Minuten 	Einer
uhrminz:	.equ	2fdfh		;		Zehner
uhrstde:	.equ	2fe0h		; Stunden	Einer
uhrstdz:	.equ	2fe1h		;		Zehner
uhrklte:	.equ	2fe2h		; Kalendertag	Einer
uhrkltz:	.equ	2fe3h		;		Zehner
uhrwtl:	.equ	2fe4h		; Wochentag
uhrwtu:	.equ	2fe5h		;
uhrmone:	.equ	2fe6h		; Monat	Einer
uhrmonz:	.equ	2fe7h		;		Zehner
uhrjare:	.equ	2fe8h		; Jahr	Einer
uhrjarz:	.equ	2fe9h		;		Zehner
;
buff_asc:	.equ	2feah 	; Uhr-ASCII als String abgelegt
;
stack		.equ 	2f00h		; 
usrstk	.equ	2f70h		;
; 
update	.equ 	2e56h		; aktuelle Cursorposition
;
srcbeg	.equ 	2fbfh		; Startadresse
srcend	.equ 	2fc1h		; Endadresse
dstbeg	.equ 	2fc3h		; Anfang Zieladresse/Parameter
dstend	.equ 	2fc5h		; 
rambeg	.equ 	2800h		;
;
vidram	.equ 	3000h		; Bildschirmadresse oben links
home		.equ 	3140h		; oben rechts (Uhrzeit)
linz		.equ 	3730h		; unten links
					; Start Parameter
vidend	.equ 	3780h		; unten rechts
;
bcup		.equ 	3140h		;
deup		.equ 	3780h		;
hlup		.equ 	3190h		;
bcdwn		.equ 	377fh		;
dedwn		.equ 	313fh		;
hldwn		.equ 	372fh		;
;
; Speicherzellen im RAM
;					 Inhalt
reg_a		.equ	2fa7h		;  Reg A
reg_b		.equ	2fa8h		;  Reg B
reg_c		.equ	2fa9h		;  Reg C
reg_d		.equ	2faah		;  Reg D 
reg_e		.equ	2fabh		;  Reg E 
reg_f		.equ	2fach		;  Reg F 
reg_h		.equ	2fadh		;  Reg H
reg_l		.equ	2faeh		;  Reg L
reg_sp	.equ	2fafh		;  Reg Stackpointer & 
					;	 Programmpointer
;
dspbuf	.equ	2fb2h		; DSPBUF LED-Zeile
dig1		.equ	2fb3h		; DIG1
dig2		.equ	2fb4h		; DIG2
dig3		.equ	2fb5h		; DIG3
dig4		.equ	2fb6h		; DIG4
dig5		.equ	2fb7h		; DIG5
dig6		.equ	2fb8h		; DIG6
dig7		.equ	2fb9h		; DIG7
data		.equ	2fbah		; Daten
adres		.equ	2fbbh		; Adresse HI
adrhi		.equ	2fbch		; Adresse LO
modflg	.equ	2fbdh		; Betriebsarten 
;
fctbuf	.equ	2fbeh		; FCT-Nr.
regnr		.equ	2fc7h		; Reg-Nr.
ioflg		.equ	2fc8h		; I/O-Parameter
nxtbuf	.equ	2fc9h		; FCT-Eingabe
maxbuf	.equ	2fcah		; max. Anz.
keybyt	.equ	2fcbh		; Key true./.invers
ciflg		.equ	2fcch		; Flag f. CI-Routine
hibuf		.equ	2fcdh		; I/O-Hilfsroutine
hbuf		.equ	2fcfh		;
;
hibuf2	.equ	2fd0h		; CTE-Sprungziele
hbuf2		.equ	2fd1h		;
brkpt1	.equ	2fd3h		; BRK-Adresse 	#1
hbufe		.equ	2fd4h		; HBUFFER fuer ESC
opcod1	.equ	2fd5h		; OpCcode-Buf	#1	
brkpt2	.equ	2fd6h		; BRK-Adresse	#2
hbuf3		.equ	2fd7h		; 
opcod2	.equ	2fd8h		; OpCode-Buf	#2
;
; Sprungtabelle fuer Funktionstasten
;
fct_7		.equ	2e20h		;
fct_6		.equ	2e22h		;
fct_5		.equ	2e24h		;
fct_4		.equ	2e26h		;
fct_3		.equ	2e28h		;
fct_2		.equ	2e2ah		;
fct_1		.equ	2e2ch		;
fct_97	.equ	2e2eh		; 87er Hardware
fct_77	.equ	2e30h		; 87er Hardware
fct_del	.equ	2e32h		;
fct_pl	.equ	2e34h		;
fct_ss	.equ	2e36h		;
fct_0		.equ	2e38h		;
fct_9 	.equ	2e3ah		;
fct_8		.equ	2e3ch		;
fct_87	.equ	2e3eh		; 87er Hardware

;
; Buffer adressen
;
batbuf	.equ	2d00h		;
fdcbuf	.equ	2f70h		; bzw. UserStack
cpybuf	.equ	8000h		;
;
; Interuptsprugverteiler
;
rst3_m:	.equ	2f8fh		;
rst4_m:	.equ	2f92h		;
trap_m:	.equ	2f8ch		;
rst5_m: 	.equ	2f95h		;
rst55_m: 	.equ	2f98h		;
rst6_m: 	.equ	2f9bh		;
rst65_m: 	.equ	2f9eh		;
rst7_m: 	.equ	2fa1h		;
rst75_m: 	.equ	2fa4h		;
;
; zum Video-Monitor
;
kalt_v: 	.equ  1000h		; kalt: Videomonitor Kaltstart
co_v:		.equ	1006h		; Consolenausgabe (C) in Position UPDATE
ci_v: 	.equ	1009h		; Tastaurabfrage  (A) Tastencode
csts_v: 	.equ	100ch		; Tastaturstatus  Z=0 Taste gedrckt, Z=1 keine Taste
hxtest_v:	.equ	1015h		; hxtest: Videomonitor ASCII/HEX-Umsetzung
clrvid_v	.equ	102dh		; clrvid: Bildschirm loeschen
clrlin_v:	.equ	1030h		; clrlin: Videomonitor Bildzeile loeschen
crspos_v:	.equ	1033h		; crspos: Videomonitor Cursor positionieren
crsein_v	.equ	1036h		; cursor einschalten
crsaus_v	.equ	1039h		; cursor ausschalten
adrot_v: 	.equ	103ch		; adrot: Videomonitor (H&L) ausgeben
;
; zum FDC
;
fdck: 	.equ	2000h		; FDCK: FDC-Routinen Kaltstart
fdcw: 	.equ	2003h		; WARM: FDC-Routinen Warmstart
;
; zum Thermodrucker
;
thermo: 	.equ	2014h		; Sprung nach Adr 2014h Thermodrucker 
;
;********************************************************************
;
	.org	0000H	; ROM-Platz #0 auf CPU-Karte
	
rst0:		jmp	kaltst	; RST0	--> Kaltstart
warmst:	jmp	warm		;	--> Warmstart
fquarz:	.db  	06h		; Taktfrequenz in MHz 
timprl:	.db  	04h		; Prellzeit der Tastatur in ms
rst1:  	jmp 	rst1_m	; RST1
dely1:	jmp 	dely1_m	; DELY1 --> 1ms Laufzeit
prtbdr:	.db  	00h		; Baudrate fuer Drucker
		.db  	48h		; 4800 Bd
rst2:		jmp 	rst2_m	; RST2
dely100: 	jmp 	dely100_m	; DELY100 --> mit SSG-Anzeige
casbdr:	.db  	00h		; Baudrate fuer Cassetten
		.db  	12h		; 1200Bd
rst3:		jmp 	rst3_m	; RST3
onesec: 	jmp 	onesec_m	; ONESEC --> 1s Laufzeit
v24bdr:	.db  	00h		; Baudrate fuer V24-Schnittstelle
		.db  	48h		; 4800Bd
rst4:		jmp 	rst4_m	; RST4
		.db  	00h		; 
trap:		pchl			; TRAP	--> FDC-Routinen
		jmp 	trap_m	;
rst5:		jmp 	rst5_m	; RST5  
		nop			;
rst55:	jmp 	rst55_m	; RST5.5 --> FDC-Routinen
		nop			;
rst6:		jmp 	rst6_m	; RST6
		nop			;
rst65:	jmp 	rst65_m	; RST6.5 
		nop			;
rst7:		jmp 	rst7_m	; RST7
		nop			;
rst75:	jmp 	rst75_m	; RST7.5 
		.db  	55h		; Monitorversion 5.5
adrset:	jmp 	adrse		; Adresse laden
ci:		jmp 	ci_v		; CI ASCII-Tastatur abfragen (A)
ri:		jmp 	ri_m		; RI ein Byte von Kasetteninterface empfangen (A)
co:		jmp 	co_m		; CO Inhalt(C) ASCII an Bildschirm (UPDATE)
po:		jmp 	po_m		; PO Inalt (C) auf Kasetteninterface (1200Bd)
lo:		jmp 	lo_m		; LO Inalt (C) ASCII-Zeichen an serielle Scnitstelle (4800Bd)
csts:  	jmp 	csts_v	; CSTS Status ASCII-Tastatur gedrueckt Z=0 sonst Z=1
csinit:	jmp 	csinit_m	; 
loinit:	jmp 	lonit_m	; LO-Init
maxram:	jmp 	maxram_m	; MEMCHK Speichertest (AB)max Adressen
prtmod:	.db  	15h		; Betriebsart (8bit,1Stop,ohne Paritaet, Takt/16)
casmod:	.db  	10h		; Cassettenbetriebsart (8daten, ohne Paritaet)
v24mod:	.db  	15h		; Betriebsart (8bit,1Stop,ohne Paritaet, Takt/16)
time:		jmp 	ti_m		; TI Uhrzeit lesen ASCII in Buffer 
v24ini:	.dw  	v24ini_m	; wird ueber PCHL aus Videomonitor angesprungen
		.db  	0e9h		; = PCHL
cnvbyt: 	jmp 	cnvbyt_m	; SSG-Umsetzen
reles:	jmp 	reles_m	; HEX-Tastatur auf Loslassen warten
press:  	jmp 	press_m	;  "           auf Drcken warten
hhkey:	jmp 	hhkey_m	; HEX-Input
cckey:  	jmp 	cckey_m	; CMD-Input
datset: 	jmp 	datset_m	; Daten-Mode einstellen
writur:	jmp 	writu_m	; WRITUR Uhrzeit stellen (HL)Std,min, (BC)Wochentag, (DE)Monat,Jahr
readur:	jmp 	readur_m	; READUR Uhrzeit lesen BCD(HL)Adresse 
save:   	jmp 	save_m	; Register vorbereiten/sichern 
restor:	jmp 	restor_m	; Registersatz laden, Start
fill:		jmp 	fill_m	; FILL Speicher fuellen (SRCBEG,SRCEND,DSTBEG)
copy: 	jmp 	copy_m	; COPY Speicher kopieren (SRCBEG,SRCEND,DSTBEG)
casot:	jmp 	casot_m	; Casetten Ausgabe
casin:	jmp 	casin_m	; Eingabe
exchan:	jmp 	exchan_m	; Copy mit Adressumsetzung
ezudsp:	jmp 	ezudsp_m	; Uhr in SSG-Anzeige
dspacc:	jmp 	dspacc_m	; (ACC) darstellen
dsphl:	jmp 	dsphl_m	; (HL) darstellen
dsperr: 	jmp 	dsperr_m	; Fehleranzeige
;
; fct-sprungverteiler
;
fctab:	jmp 	fctcot	; FCT-0 = Casettenausgabe
		jmp 	fctcin	; FCT-1 = Casetteneingabe
		jmp 	fctsod	; FCT-2 = CPU SOD
		jmp 	fctsid	; FCT-3 = CPU SID
		jmp 	fcthex	; FCT-4 = Thermodrucker Hexdump
		jmp 	fctasc	; FCT-5 = Thermodrucker ASCII
		jmp 	fctxch	; FCT-6 = Exchange 
		jmp 	fctins	; FCT-7 = Insert
		jmp 	fctwur	; FCT-8 = Write Uhr
		jmp 	fctrur	; FCT-9 = Read Uhr
		jmp 	fctfil	; FCT-A = Fill
		jmp 	fctbrk	; FCT-B = Breakpoint
		jmp 	fctcpy	; FCT-C = Copy
		jmp 	fctdel	; FCT-D = Delete
		jmp 	fctstp	; FCT-E = Single-Step
		jmp 	ledasc	; MOVID-Sprungziel: LEDASC 
;		
; Ende der Sprungtabelle	
;
;--------------------------------------------------------------------
;
; Aufbau Mode-Flag
;
;  Bit7 6 5 4 3 2 1 0
;	I x x x I I I I
;     I       I I I + - Daten-Mode
;     I       I I + - - Adress-Mode
;     I       I + - - - Register-Mode
;     I       + - - - - Funktionseingabe
;	+ - - - - - - - - Einzelschrittbetrieb
;
;	*) Bit 4-6 fuer ESC-Routinen 
;	   siehe CO_m
;
kaltst:	lxi	sp,stack	;
		call	save		; Register vorbereiten/sichern
		call	init		; Init
		lxi	h,keybyt	; Key true oder invers 
		mvi	m,0ffh	;     
		in	comand	; Test auf Hex-Tastatur
		cpi	7fh		; Diode auf Bit 7
		jz	mainhex	; ja HEX-Tastatur
		inr	m		; RAM 2FCBh auf 0h setzen
		cpi	80h		; GE-Diode in HEX-Tastatur
		jz	mainhex	;
		cpi	00h		; ASCII invers
		jz	kalt_v	; zum Videomonitor
		dcr	m		; KEYBYT FFh setzen
		jmp	kalt_v	; zum Videomonitor
;	
		call updhex		; ! hieryu gibt es keinen Einsprung
;
warm:  	xra	a		;
		out	fdc_sel	; FDC Laufwerk A einstellen
		lxi	sp,stack	;
mainhex:	call	hexkey	; hauptschleife Main-Hextastatur
		call	ssgdsp	; 
		call	cmdkey	;
		jmp	mainhex	; 
;
; INIT 	nimmt die Grundeinstellung vor
;	
init:  	lxi	h,usrstk	; Anwenderstack
		mov	a,l		; sortieren fuer REGBUF
		mov	l,h		; H&L tauschen
		mov	h,a		;
		shld	reg_sp	; Stackpointer retten
		mvi	a,00h		;
		sta	0000h		; ! bringt im EPROM nichts
		mvi	a,00h		;
		sta	modflg	; Flags loeschen
		sta	ioflg		; IO-Byte
		call	adrfix	;
		call	dat		;
		mvi	a,00h		;
		sta	regnr		; REG A voreinstellen
		out	ssm_ff	; Einzelschrittmodul FlipFlop
		out	40h		; FDC Selektbyte
		out	led_zeil	; LED
		out	prom_ctl	; Promer
		mvi	a,0ch		;
		out	therm_d	; Thermodrucker
		mvi	a,20h		;
		out	cs_mode	; Serielle Schnittstelle
		mvi	a,82h		;
		out	par_ctl	; Parallele Schnittstelle
		mvi	a,0f0h	;
		out	prom_adrh	; Promer LEDausschalten
		mvi	a,0f4h	;
		out	uhr_ctl	; Ruhepegel fuer Uhr
		mvi	a,17h		; RSTs sperren
		sim			; Interupptmaske
		nop			; wg. NSC800 JR-Befehl
		in	fdc_dat	;
		out	cs_bell	; Summer
		ret			;
;
; SAVE 		rettet alle Register
;
save_m:	sta	reg_a		; Register sichern/vorbereiten 
		mov	a,l		; H mit L tauschen
		mov	l,h
		mov	h,a
		shld	reg_h		; HL sichern 
		push	psw
		pop	h		; PSW in HL
		mov	a,l
		sta	reg_f		; PSW sichern
		lxi	h,reg_b	; Zeiger auf REG-Buffer
		mov	m,b		; REG B 
		inx	h
		mov	m,c		; REG C
		inx	h
		mov	m,d		; REG D
		inx	h
		mov	m,e		; REG E
		lxi	h,0000h	; SP vorbereiten
		dad	sp
		inx	h
		inx	h
		mov	a,l		; (H&L) tauschen
		mov	l,h
		mov	h,a
		shld	reg_sp	; SP sichern
		ret			;
;
; HEXKEY 	fragt HEX-Tastatur ab und schreibt
;		Eingabe ins Daten oder Adressfeld
;
hexkey:	call	hhkey_m	; HEX Tastenzeile einlesen
		rnc			; keine Taste RET
		call	shift		; Eingabe nachschieben
		call	convt		; HEX/SSG-Umsetzung
		call	reles_m	; auf Loslassen warten
		ret
;
; HHKEY_m	fragt beide HEX-Zeilen ab
;		einschliesslich entprellen
;		no key	= Zero-Flag on (Z)=1
;		key		= Carry-Flag on (CY)=1
;				Key-Nr. in (A) und (C)
;
hhkey_m:	call hkey		; erste Abfrage
		rnc			; keine HEX-Taste gedrckt
		mov	b,a		; Taste in (B) retten
		call	dely7m	; Prellzeit abwarten
		call	hkey		; Abfrage wiederholen
inpcmp:	cmp	b		; Taste noch gedrueckt ?		
		jz	setcy		; gleich (CY)=1
		stc			; ungleich (CY)=0
		cmc			;
		ret			;
					;
setcy:  	stc			;
		ret
;
; HKEY	wie HHKEY, ohne Entprellung
;
hkey:		in	hexlo		;
		call	keyin		; Input
		rc			; Tasteaktiv
		in	hexhi		; Taste 8-f abfragen
		mvi	c,08h		; Anfangswert fuer 2.Zeile
		call	keyin2	; Input
		ret			;
;
; KEYIN	ermittelt bei aktiver Taste die Bit-Nr.
;		no key	= Zero-Flag on (Z)=1
;		key		= Carry-Flag on (CY)=1
;				Key-Nr. in (A) und (C)
;		
keyin:  	mvi	c,00h		;
keyin2: 	lxi	h,keybyt	;
		xra	m		;
		rz			;
					;
keylop: 	rrc			; High-Bit ins CY
		jc	keyend	; fertig
		inr	c		; Bit-Zaehler erhoehen
		jmp	keylop	; weiter
keyend:	mov	a,c		; Ergebnis in (A)
		ret			;
;
; SHIFT	schiebt HEX-Eingabe ins Daten oder Adressfeld
;
shift:  	lxi	h,data	; Zeiger auf Datenfeld
		lda	modflg	; Mod-Flag holen
		rrc			; ins Carry-Bit
		jc	sftdat	; Datenmode
		rrc			;
		jc	sftadr	; Adressmode
		ret			
;	
sftadr: 	rrc			; Regidter Flag testen
		jc	sftreg	; Registermode
		inx	h		; Pointer aufAdressfeld
		call	nibadd	;
		mov	a,m		; ADRLO erneut holen
		mov	m,c		; aufadirtes Byte noch ADRLO
		ani	0f0h		; 4LSB loeschen
		rrc
		rrc
		rrc
		rrc
		mov	c,a		; fuer NIBADD ablegen
		inx	h		; Zeiger auf ADRHI
sftdat: 	call	nibadd	; Nibble nachschieben
		mov	m,c		; 
		lda	modflg	;
		mov	b,a		; Mod-Flag retten
		ani	08h		; FCT-Flag maskieren
		rz			; kein FCT-Mode, fertig
		mov	a,b		;
		ani	0feh		; DAT-Flag loeschen
		ori	02h		; ADR-Flag setzen
		sta	modflg	; Modflag sichern
		ret
;
sftreg:  	mov	a,c
		adi	06h		; verschieben A=0
		ani	07h		; MSB loeschen
		sta	regnr		; sichern
		ret
;
; NIBADD	schiebt das eingegebene Nibble in das von
;		(HL) adressierte Byte
;		Ergebnis in (C)
;
nibadd: 	mov	a,m		; altes Byte holen
		ani	0fh		; obere Haelfte loeschen
		rlc
		rlc
		rlc
		rlc
		ora	c		; neues Nibble anhaengen
		mov	c,a		; sichern
		ret
;
; READOT	aktualisiert das Datenfeld
;		entsprechend Adressfeld bzw. Reg-Nr.
;
readot: 	lda	modflg	; Mod-Flag holen
		mov	b,a		; retten
		ani	08h		; FCT-Flag maskieren
		rnz			; FCT-Mod, fertig
		mov	a,b		; Mod-Flag zurueckholen
		ani	04h		; REG-Flag maskieren
		jnz	readrg	; Reg-Mode, fertig
		lhld	adres		; Adressfeld lesen
readat: 	mov	a,m		; Daten holen
		sta	2fbah		; ins Datenfeld bringen
		ret
;
readrg: 	lxi	h,reg_a	; Anfang REGBUF
		lda	regnr		; Reg-Nr. holen
		mov	e,a
		mvi	d,00h		; Versatz erzeugen
		dad	d		; (HL)= Regbuf
		jmp	readat
;
; WRITIN	schreibt die Daten aus dem Datenfeld ins RAM
;		bzw. REGBUF
;
writin: 	lda	modflg	; Mod-Flag holen
		ani	04h		; REG-Flag maskieren
		jnz	writrg	; Regmode, fertig
		lhld	adres		; Adressfeld auslesen
writda: 	lda	data		; Daten holen
		mov	m,a		; und ablegen
		ret
;
writrg: 	lxi	h,reg_a	; Anfang REGBUF 
		lda	regnr		; Register holen
		mov	e,a
		mvi	d,00h		; Versatz erzeugen
		dad	d		; (HL)= Zieladresse
		jmp	writda	;
;
; CONVT	setzt die Hex-Daten aus dem Daten- Adressfeld in den
;		SSG-Code um und ueberschreibt diesen in den Diplaybuffer
;
convt:  	lda	data		; Daten holen
		mov	b,a		; retten
		lda	modflg	;
		ani	80h		; STP-Flag maskieren
		jz	cnvled	; kein Singelstep, fertig
		lda	reg_f		; Flag-Register holen
		mov	b,a		; retten
cnvled: 	mov	a,b		; LED-Byte holen
		sta	dspbuf	; LED-Zeile
		lxi	d,dig1	; Anzeige Digit 1
		lda	data		; Datenfeld wieder holen
		call	cnvbyt_m	; SSG-Umsetzung von Data
		lxi	d,dig4	; Anzeige Digit 4
		lda	modflg	; Modflag holen
		ani	04h		; REG-Mod ?
		jnz	cnvreg	; ja, weiter
cnvadr:	lda	adres		;
		call	cnvbyt_m	; ADR-LO nach SSG
		lda	adrhi
		call	cnvbyt_m	; ADR-HI nach ssg
		jmp	dpset		;
					
cnvreg: 	lda	regnr		; REG-Nr. holen		
		lxi	h,regtab	; REG-SSG-Code
		call	cnvsg2	; REGNR nach SSG
		xra	a
		stax	d		; DIG5 loeschen
		inx	d
		stax	d		; DIG6 loeschen
		inx	d
		stax	d		; DIG7 loeschen
;
; DPSET	Dezimalpunkt ins Daten- oder Adressfeld  einblenden
;
dpset:  	lxi	d,dig1	; Pointer auf Digit 1
		lda	modflg	; Modflag holen
		ani	01h		; DAT-Flag maskieren
		jnz	dpcte		; Datenmode, weiter
		inx	d
		inx	d
		inx	d		; Zeiger auf Digit 4
dpcte:  	ldax	d		; SSG Code holen
		ori	80h		; MSB setzen (=Dezimalpunkt)
		stax	d		; SSG-Code ablegen
		ret
;
; CNVBYT_m 
;		setzt (A) in SSG-Code um
;		und schreibt ihn die von D&E adr. RAM-Zelle
;		erhoeht (DE) um 1
;
cnvbyt_m:	mov	b,a		; Byte retten
		call	cnvssg	; SSG-Code holen
		mov	a,b		; Byte zurueckholen
		rrc
		rrc
		rrc
		rrc
		call	cnvssg	; SSG-Code holen
		ret
;
; CNVSSG	Setzt (A) unteres Nibble in SSG-Code um 
;		und schreibt ihn die D&E adr. RAM Zelle
;		(DE) um 1 erhoehen
;
cnvssg: 	lxi	h,segtab	; Tabellenadresse laden
cnvsg2:  	push	d		; Bufferzeiger retten
		mvi	d,00h		; Versatz bilden
		ani	0fh		; oberes Nibble loeschen
		mov	e,a		;
		dad	d		; (HL)=Code-Adresse
		pop	d		; Pointer wieder zuerueck holen
		mov	a,m		; SSG-Code holen
		stax	d		; und in DSP-Buffer schreiben
		inx	d		; Zeiger auf naechste Adresse
		ret
;
; SEGTAB	Tabelle der Siebensegmet-Code von 0..9
;		A..F und HLPSP 
;
segtab:	.db	3fh		; 0
		.db	06h		; 1
		.db	5bh		; 2
		.db	4fh		; 3
		.db	66h		; 4
		.db	6dh		; 5
		.db	7dh		; 6
		.db	07h		; 7
		.db	7fh		; 8
		.db	6fh		; 9
regtab:	.db	77h		; A
		.db	7ch		; b
		.db	39h		; C
		.db	5eh		; d
		.db	79h		; E
		.db	71h		; F
		.db	76h		; H
		.db	38h		; L
		.db	6dh		; S
		.db	73h		; P
;
; SSGDSP	Ueberschreibt den SSG-Code aus dem
;		Display-Buffer in die Anzeige
;		feste Laufzeit von 10ms
;
ssgdsp: 	lxi	d,dig7	; oberste Ram-Zelle in DSPBUFFE
		inx	d
		lxi	h,hibuf	; Output-Routine im RAM
		mvi	m,0d3h	; HEX-Code "OUT"
		inx	h
		inx	h
		mvi	m,0c9h	; HEX-Code "RET"
		dcx	h
		mvi	c,outpor	; OUT-Portadresse + 1
ssglop: 	dcx	d		; Adresse in DSP-Buffer
		mov	a,c		; alte Portadresse
		sui	10h		; Portadresse erniedrigen
		ora	a		; Flags setzen
		mov	c,a		; Outadresse retten
		mov	m,a		; Outadresse in OUT-Routine
		ldax	d		; SSG-Code aus DSPBUF holen
		call	hibuf	;
		call	dely1	; 1Khz MUX-Frequenz
		jnz	ssglop	; 8 Einzelausgaben abwarten
		call	dely2m	; auf 10ms verlaengern
		ret
;
; CMDKEY	fragt die Befehlstasten ab und
;		verzweigt nach Sprungtabelle
;
cmdkey: 	call cckey		; CMD-Key abfragen
		rnc			; keine Taste, fertig
		lxi	h,jmptb	; CMD-Sprungverteiler
		mov	b,a		; Input retten
		lda	modflg	; Mod-Flag holen
		ani	08h		; FCT-Flag maskieren
		jz	branch	; FCT-Mode, Sprung
		mov	a,b		; FCT-Mode, Input holen
		cpi	nxtnr		; NXT? (naechster Parameter)
		jz	branch	; ja, Sprung
		cpi	adrnr		; ADR? (zurueck zum Adressmode)
		jz	branch	; ja, Sprung
		cpi	fctnr		; FCT? (zuerueck zum FCT-Feld)
		jz	branch	; ja, Sprung
		cpi	runr		; RUN?
		rnz			; nein, dann RET
					;
brnch:  	lda	modflg	; LEDASC-Sprungziel
		ani	0f7h		; FCT-Flag loeschen
		sta	modflg	; sichern
		lxi	h,fctab	; FTC-Sprungverteiler
		lda	data		;
		ani	0fh		; 4 MSB loeschen
		mov	b,a		; FCT-Nr. 0...F
branch: 	mov	a,b		; Input holen
		rlc			; Input mal 2
		add	b		; Input mal 3
		mov	e,a		;
		mvi	d,00h		; Versatz in (DE)
		dad	d		; (HL)= Tabellen-Adress
		lxi	d,retur	;  SUB-Ruecksprungadress 032BH
		push	d		; in Stack sichern
		pchl			; Verzweigung ueber Tabell
retur:	call	reles_m	; CMD-loslassen abwarten
		ret			; zurueck zum Monitor
;
; CCKEY	fragt die CMD-Zeile ab mit Entprellung
;		und unterdruckt die Dioden-Nr.
;		no Key	Zero-Flag on (Z)=1
;		Key		Carry-Flag on (CY)=1
;		Key-Nr. 	in (A) und (C)
;
cckey_m: 	call	ckey		; erste Abfrage
		rnc			; keine CMD-Taste gedrueckt, fertig
		mov	b,a		; Input in B retten
		call	dely7m	; Prellzeit
		call	ckey		; Abfrage wiederholen
		jmp	inpcmp	; Abfrage vergleichen (in HHKEY)
;
; CKEY	wie CCKEY, ohne entprellen
;
ckey:  	in	comand	;	 CMD-Taste abfragen
		call	keyin		; Input-Subroutine
		rnc			; keine Taste, fertig
		cpi	diodnr	;
		jnz	setcy		; nein, CY on
		stc			; ja, CY off
		cmc
		ret
;
; Sprungtabelle fuer CMD
;
jmptb:	jmp	nxt		;
		jmp	bst
		jmp	fct
		jmp	reg
		jmp	adr
		jmp	dat
		jmp	run
;
; Mode-Keys DAT, ADR/FCTF, REG, FCT
;
datset_m:	shld	adres		; HL ins Adressfeld
dat:  	lda	modflg
		ani	0fdh		; ADR-Flag loeschen
		ori	01h		; Dat-Flag setzen
		jmp	modcte
		;
adrfix: 	lxi	h,rambeg	; Anfangsadresse RAM
adrse:  	shld	adres		; ins Adressfeld laden
adr:  	mvi	a,08h		; ein Strich
		sta	dig3		;
		lda	modflg
		ani	0f2h		; DAT,REG,FCT-Flag loeschen
		ori	02h		; ADR-Flag setzen
		jmp	modcte	;
		;
reg:  	mvi	a,48h		; zwei Striche
		sta	dig3		;
		lda	modflg
		ani	0feh		; DAT-Flag loeschen
		ori	06h		; ADR, REG-Flag setzen
		jmp	modcte	;
		;	
fct:  	mvi	a,49h		; drei Striche
		sta	dig3		; FCT-Mode kennzeichen
		mvi	a,07h		; max fuer 4Parameter
		sta	maxbuf	;
		xra	a
		sta	nxtbuf	; NXT-Buffer loeschen
		mvi	a,0ffh
		sta	data		;
		lxi	h,0000h
		shld	adres		; 0000 ins Adressfeld
		lda	modflg
		ani	0f9h		; ADR, REG-Flag loeschen
		ori	09h		; FCT, DAT-Flag setzen
modcte: 	sta	modflg
updhex: 	call	readot	; Daten ins Datenfeld
		call	convt		; SSG umsetzen
		ret
;
; NXT		erhoeht ADR/Reg und schreibt ins Datenfeld
;
nxt:  	lda	modflg	;
		ani	08h		; FCT-Flag maskieren
		jnz	nxtcte	; FCT-Mode ?, ja weiter
		call	writin	; Daten schreiben
		jnz	nxtreg	; REG-Mode ?, ja weiter
		inx	h		; Adressfeld hochzaehlen
		shld	adres		; und sichern
		jmp	updhex	; Anzeige aktualisieren
		;
nxtreg:	lda	regnr		;
		inr	a		; Reg-Nr + 1
		daa			; Dezimalkorrektur
nxtsto:	ani	0fh		; uebertrag unterdruecken
		sta	regnr		; neue Reg-Nr ablegen
		jmp	updhex	; Anzeige aktualisieren
					;
nxtcte:	lxi	h,maxbuf	; maximalwert fuer Eingabe
		lda	nxtbuf	; Eingabe-Zaehler
		cmp	m		; Maximum erreicht
		rnc			; ja, ende
		lhld	adres		; letzter Parameter
		xchg			; nach DE retten
		lxi	h,srcbeg	; Anf. Parameter-Buffer
		mov	c,a		; Versatz erzeugen
		mvi	b,00h
		dad	b		; SRCBEG + NXTBUF
		mov	m,e		; Parameter in Buffer ablegen
		inx	h
		mov	m,d
		lxi	h,0000h		
		shld	adres		; Adressfeld wieder loeschen
		lxi	h,nxtbuf	; Eingabezaehler + 2
		inr	m
		inr	m
		jmp	updhex	; adr.=0000 anzeigen
;
; BST	schaltet Adr./Reg. zurueck
;	
bst:		call	readot	; Zeiger ermitteln
		jnz	bstreg	; Reg-Mode, weiter
		dcx	h		; Adressfeld - 1
		shld	adres		; und ablegen
		jmp	updhex	; Anzeige aktualisieren
					;
bstreg:	lda	regnr		; REG-Nr. holen
		dcr	a		; - 1
		jp	nxtsto	; und ablegen
		mvi	a,09h		; auf Anfangswert setzen
		jmp	nxtsto	; neue REG-Nr ablegen
;
; RUN	Programmstart bei der im Adressfeld stehende Adresse
;	laed vorher alle Register inkl.Flags
;
run:  	xra	a		; (A) loeschen
		sta	dspbuf	; LED's dunkel schalten
		call	reles_m	; auf loslassen warten
					;
restor_m:	lda modflg
		ani	80h		; STP-Mode ?
		cnz	rst1		; ja: Breakpoint setzten
		lda	reg_f		; Flags holen
		mov	c,a
		push	b		; aufStack sichern
		pop	psw		; und ins CPU-Flag-Register
		lhld	reg_sp	; Anwenderstack holen
		mov	a,l		; sortieren
		mov	l,h
		mov	h,a
		sphl			; in CPU-STACK
		lxi	h,reg_b	; REB B laden
		mov	b,m		;
		inx	h		;
		mov	c,m		; REG C laden
		inx	h		;
		mov	d,m		; REG D laden
		inx	h		;
		mov	e,m		; REG E laden
		lhld	adres		; Startadresse holen
		push	h		; im Stack sichern
		lhld	reg_h		; REG H laden
		mov	a,l		; sortieren
		mov	l,h		;
		mov	h,a		;
		lda	reg_a		; Reg A laden
		ret			; Programmstart
;
; RELES_m	wartet auf loslassen einer aktiven Taste
;		aktiviert in dieser Zeit die SSG-Anzeige
;
reles_m: 	call	rls		; Abfrage unterprogramm
		jc	reldsp	; Taste bettigt, weiter
		call	dely7m	; Prellzeit
		call	rls		; erneute Abfrage
		rnc			; keine Taste,ende
reldsp: 	call	ssgdsp	; SSG-Anzeige aktivieren
		jmp	reles_m	; neuer Abfragezyklus
	
rls:  	call	hkey		; HEX-Keys abfragen
		rc			; HEX aktiv, zuruck
		call	ckey		; CMD-Keys abfragen
		ret			;
;
; PRESS_m	Springt bei Tastendruck zurueck zum Warmstart
;	sonst Return
;
press_m: 	call	dely100_m	; Anzeige aktivieren
		call	hhkey_m	;
		jc	preles	; HEX-Key aktiv, weiter
		call	cckey_m	;
		rnc			; keine Taste, ende
		;
preles: 	call	reles_m	; auf loslassen warten
		call	adrfix	; Adr= RAM-Beginn
		jmp	warmst
;
;====================================================================
;
; Laufzeitprogramme
;	
onesec_m:	push	psw		; 1s warten
		mvi	a,0ah		; 10d
onelop: 	call	dely100_m	;	
		dcr	a
		jnz	onelop
		pop	psw
		ret
;
dely100_m:	push	psw		; 100ms warten
		mvi	a,0ah
delylp: 	push	psw
		push	h
		push	d
		push	b
		call	ssgdsp
		pop	b
		pop	d
		pop	h
		pop	psw
		dcr	a
		jnz	delylp
		pop	psw
		ret
;
dely7m: 	push	psw		; Tastaturprellzeit
		lda	timprl
dlop:   	call	dely1		; 7 ms warten
		dcr	a
		jnz	dlop
		pop	psw
		ret	
;	
dely2m:	call	dely1		; 2 ms warten
dely1_m: 	push	psw		; 1ms warten
		lda	fquarz
		add	a
		add	a
		inr	a
		add	a
		add	a
		add	a
delop:  	dcr	a
		jnz	delop
		pop	psw
		ret	
;
; Freiraum von 04BF bis 04FF
; 
;********************************************************************
; 
; STP/RAM Einzelschritt und RAM-Funktion
;
; Drei Betriebsarten mglich:
;
;	1. Einzel-Byte-Betrieb
;	   Mit Einzelschrittmodul; RAM oder ROM
;
;	2. Breakpointbetrieb
;	   Softwarehaltepunkt im RAM setzen
;
;	3. Einzelschritt-Betrieb
;	   Single-Step im RAM
;
;
	.org 0500h
	
fctbyt	lhld	adres		; Startadresse holen
		mvi	a,40h		; BYT-Flipflop setzen
		out	ssm_ff	; (auf STP-Modul
		pchl			; Byte-Mode starten
;
; Breakpoint-Betrieb im RAM
;
fctbrk: 	lxi	h,warmst	; Warmstartadresse
		call	brkini	; Breakpoint vorbereiten
		lhld	adres		; Breakadresse
		shld	brkpt1	; retten
		mov	a,m		; OPCODE holen
		sta	opcod1	; sichern
		mvi	m,0d7h	; RST2 einsetzen
		lda	reg_f		; Restor fuer FCT-B
		mov	c,a		; REG C = CPU-Flags
		push	b
		pop	psw		; CPU Flags setzen
		lhld	reg_sp	; Userstack Pointer
		mov	a,l		; sortieren
		mov	l,h
		mov	h,a
		sphl			; in CPU Stack-Pointer
		lxi	h,reg_b	; Zeiger auf REG-Buffer
		mov	b,m		; REG B laden
		inx	h
		mov	c,m		; REG C laden
		inx	h
		mov	d,m		; REG D laden 
		inx	h
		mov	e,m		; REG E laden
		lhld	srcbeg	; Startadresse holen
		push	h		; und sichern
		lhld	reg_h		; REG H laden
		mov	a,l
		mov	l,h
		mov	h,a
		lda	reg_a		; REG A laden
		ret			; Start bis Breakpoint
;
; Einzelschritt-Betrieb im RAM
;
fctstp: 	lxi	h,0000h	; Warmstartadresse
		lxi	d,0003h
		dad	d		; errechnen
		call	brkini	; Breakpoints vorbereiten
		ret
;
; BRKINI	Brakpoints vorbereiten
;		STP-Flag setzen
;		REG F loeschen, 
;		CTE-Adresse im Hilfsbuffer HIBUF ablegen
;			
brkini: 	shld	hbuf2		; CTE Adresse (Warmstart)
		mvi	a,0c3h	; "JMP"
		sta	hibuf2	; CTE-Adresse fuer HIBUF
		lda	modflg	; Mode-Flag holen
		ani	0f7h		; FCT-Flag loeschen
		ori	80h		; STP-Bit setzen
		sta	modflg	; sichern
		xra	a
		sta	reg_f		; REG F loeschen
		lxi	h,0000h	; Brakpoint-Adr. loeschen
		shld	brkpt1
		shld	brkpt2
		lxi	h,dig7
		mvi	m,00h
		dcx	h
		mvi	m,50h		; "r"
		dcx	h
		mvi	m,1ch		; "u"
		dcx	h
		mvi	m,54h		; "n"
		ret	
;
; REST1	ersetzt den naechsten Opcode
;		durch einen Breakpoint (RST2 = D7h)
;
rst1_m:	lhld	adres		; aktuelle Adresse holen
		call	rest11	; Adr. des naechsten OP-Code
		mov	a,m
		sta	opcod1	; und retten
		mvi	m,0d7h	; "RST2" einsetzen
;
; REST12	setzt bei den Befehlen 
;		PCHL, RST, RET, JMP bzw CALL einen zweiten
;		Breakpoint an der moeglichen Zieladresse ein
;		RST2 = D7h
;
rest12:	ldax	d		; Opcode erneut holen
		cpi	0e9h		; PCHL ?
		jnz	rstct2	; nein weiter
		lhld	reg_h		; Ziel in (HL)
		mov	a,l		; sortieren
		mov	l,h		;
		mov	h,a		;
		jmp	rstend	;
					;
rstct2: 	call	rstbef	; ist es ein RSTn ?
		jz	rstct3	; nein, weiter
		ldax	d		;
		ani	38h		; Zieladressbit maskieren
		inr	a		; Adresse hinter "C3"
		mov	l,a		;
		mvi	h,00h		; (HL) 01...39h
		mov	e,m		; Zieladresse low
		inx	h
		mov	d,m		; Zieladresse high
		xchg			; (HL) RST-Sprungziel
		jmp	rstend	;
					;
rstct3: 	call	retbef	; ist es ein RET ?
		jz	rstct4		; nein, weiter
		lhld	reg_sp	; Userstack-Pointer
		mov	a,l		; sortieren
		mov	l,h
		mov	h,a
		mov	e,m		; Stack auslesen
		inx	h
		mov	d,m		; (DE) RET-Adress
		xchg
		jmp	rstend	;
					;
rstct4: 	call	jmpbef	; ist es ein JMP/CALL ?
		rz			; nein, ende
		inx	d		; Zieladresse holen
		ldax	d
		mov	l,a		; untere Adresshaelfte
		inx	d
		ldax	d
		mov	h,a		; obere Adresshaelfte
					;
rstend: 	shld	brkpt2	; Break-Adresse 2 retten
		mov	a,m		; Opcode holen
		sta	opcod2	;
		mvi	m,0d7h	; "RST2" einsetzen
		ret
;
; REST11	ermittelt die Adressen des naechsten Opcode
;		(HL) Adresse des aktuellenOpcode
;
rest11: 	push	h		;
		xchg			; (DE) Adr. aktuellen Opcode
		mvi	b,03h		; Schrittweite fuer Dreiwort-Befehle
		call	drwbef	; ist es ein Dreiwort Befehl ?
		jnz	rstcte	; ja, weiter
		dcr	b		; Schrittweite Zweiwort-Befehl
		call	zwwbef	; ist es ein Zweiwort Befehl ?
		jnz	rstcte	; ja, weiter
		dcr	b		; Schrittweite Einwort-Befehl
rstcte: 	mov	c,b		;
		mvi	b,00h		; (BC) 1,2,3
		pop	h
		dad	b		; (HL) naechster Opcode
		shld	brkpt1	; sichern
		ret
;
zwwbef: 	lxi	h,op2tab	; Tabelle Zweiwort-Befehleg
		mvi	c,12h		; Tabellenlaenge 18d
		jmp	oplop		;
		;
drwbef: 	lxi	h,op3tab	; Tabelle Dreiwort-Befehle
		mvi	c,1ah		; Laenge 26d (Berichtigt aus HAT1)
		jmp	oplop		;
		;
jmpbef: 	lxi	h,jmptab	; Tabelle JMP/CALL-Befehl
		mvi	c,12h		; Laenge 18d
		jmp	oplop		;
					;
retbef: 	lxi	h,retab	; Tabelle Return-Befehl
		mvi	c,09h		; Laenge 9d
		jmp	oplop
		
rstbef: 	lxi	h,rstab	;  Tabelle Restart-Befehl
		mvi	c,08h		; Laenge 8
oplop:  	ldax	d		; Opcode holen
		cmp	m		; REG A = Tabwert
		jz	opend		; ja, weiter
		inx	h
		dcr	c
		jnz	oplop		; Schleife 
opend:  	xra	a		; REG A loeschen
		cmp	c		; Tab durchlaufen
		ret			; ja, (Z)=1 ende
;
; REST2	ersetzt Breakpoint durch den 
;		urspruenglichen Opcode
; 
rst2_m: 	shld	hibuf		; HL retten
		sta	hbuf		; REG A retten
		pop	h		; gleicht RST-Interupt aus
		dcx	h		; wg Einwort Interupt
		shld	adres		; aktualisierte Adresse
		lhld	brkpt1	; Breakadresse 1 holen
		lda	opcod1	; und geretteten Opcode 1
		mov	m,a		; wieder einsetzen
		lhld	brkpt2	; Breakadresse 2 holen
		lda	opcod2	; und geretteten Opcode 2
		mov	m,a		; wieder einsetzen
		lda	hbuf		; REG A laden
		lhld	hibuf	; und HL
		call	save		; saemtliche REGs in RAM-Buffer
		lxi	h,0000h	; Brakadresse loeschen
		shld	brkpt1
		shld	brkpt2
		push	h		; Eintragungen im User-Stack
		push	h		; loeschen
		pop	h
		pop	h
		lxi	sp,stack	; Monitor Stack
		jmp	hibuf2	; Warmstart
;
; Tabelle
;	
op2tab:	.db	06h		; Zweiwort-Befehle
		.db	0eh
		.db	16h		;
		.db	1eh	
		.db	26h		;
		.db	2eh	
		.db	36h		;
		.db	3eh
		.db	0c6h		;
		.db	0ceh		
		.db	0d6h		;
		.db	0deh	
		.db	0e6h		;
		.db	0eeh	
		.db	0f6h		;
		.db	0feh
		.db	0d3h		;
		.db	0dbh	
op3tab:	.db	01h		; Dreiwort-Befehle
		.db	11h		
		.db	21h		;
		.db	22h	
		.db	2ah		;
		.db	31h
		.db	32h		;
		.db	3ah
jmptab:	.db	0c2h		; JMP-Tab
		.db	0c3h	
		.db	0cah		;
		.db	0d2h	
		.db	0dah		;
		.db	0e2h		
		.db	0eah		;
		.db	0f2h
		.db	0fah		;
caltab:	.db	0c4h
		.db	0cch		;
		.db	0cdh	
		.db	0d4h		;
		.db	0dch
		.db	0e4h		;
		.db	0ech
		.db	0f4h		;
		.db	0fch
retab:	.db	0c0h		; 
		.db	0c8h
		.db	0c9h		;
		.db	0d0h		 
		.db	0d8h		; 
		.db	0e0h		 
		.db	0e8h		; 
		.db	0f0h		 
		.db	0f8h		;
rstab:	.db	0c7h		; 
		.db	0cfh		; 
		.db	0d7h		 
		.db	0dfh		; 
		.db	0e7h		 
		.db	0efh		; 
		.db	0f7h		 
		.db	0ffh		; 
;
;====================================================================
;
; RAM-Verwaltung
;
; EXCHAN	kopiert Speicherbereiche um und
;		passt dabei diejenigen Adressen
;		an, die im kopierten Block liegen
;
fctxch:	lhld	adres		;
		shld	dstbeg	; Zieladresse
		call	exchan_m	;
		lhld	dstbeg	; Zieladresse holen
		jmp	datset	; und Daten-Mode einstellen
					;
exchan_m:	call	copy		; umkopieren
		lhld	srcbeg	; Adressdifferenz bilden
		xchg	
		lhld	dstbeg	;
		shld	brkpt1	; Anfang Zielbereich
		mov	a,l
		sub	e
		mov	l,a		; Differenz low
		mov	a,h
		sbb	d
		mov	h,a		;Differenz high
		shld	dstend
		xchg
		lhld	srcend
		dad	d		; Ende Zielbereich
		shld	brkpt2	; fuer Endabfrage ablegen
exlop:  	lhld	brkpt1	; aktuellerOpcode
		shld	hibuf		; Adresse retten
		call	rest11	; naechster Opcode n.BRKPT1
		xchg			; (DE) Adr naechster Opcode)
		lhld	brkpt2	; (HL) Endadresse Zielbereich
		call	comp16	; Ende ?
		rnc			; ja,ende
		lhld	hibuf		; Adresse zurueckholen
		xchg
		call	drwbef	; ist ein Dreiwort-Befehl
		cnz	modify	; ja,weiter
		jmp	exlop		; Schleife
;
; CMPEND	vergleicht (HL) mit SRCEND
;		(HL) > SRCEND, dann Bereichsende
;		(CY) = 1
;	
cmpend: 	xchg			;(DE) RAM-Adresse
		lhld	srcend
comp16: 	mov	a,d		; 16Bit Vergleich
		cmp	h
		rc			; Adr. > SRCEND, ende
		mov	a,e
		cmp	l		; Adr. = SRCEND, (Z) = 1
		ret			; ADR. > SRCEND, (CY) = 1
;
modify: 	inx	d		; (DE) Adr des aktuellen Opcode
		ldax	d		; Absolute Adresse low
		mov	l,a
		inx	d
		ldax	d		; Absolute Adresse high
		mov	h,a		; (HL) Absolute Adresse
		call	cmpend	; Vergleich mit SRCEND
		rnc			; > als SRCEND,ende
		lhld	srcbeg	; Adressdifferenz
		call	comp16	; Vergleich mit SRCEBEG
		jz	mod		; 
		rc			; < als SRCBEG, ende
mod:  	lhld	dstend	; Adressdifferenz
		dad	d		; zur absoluten Adresse addieren
		xchg			; (DE) modifizierte Adresse
		lhld	hibuf		; aktueller Opcode
		inx	h		; zweites Byte im Befehl
		mov	m,e		; modifiz. Adresse low
		inx	h		; drittes Byte im Befehl
		mov	m,d		; modifiz. Adresse high
		ret			;
;
; INSERT	fuegt an der aktuellen RAM-Adresse
;		NOPs (=00) ein (RAM-Ende beachten)
;		keine Adressanpassung
;
fctins: 	lda	adres		; Anzahl holen
		push	psw
		mov	e,a
		mvi	d,00h
		lhld	srcbeg
		push	h
		dad	d		; Zieladresse
		shld	dstbeg
		lxi	h,buff_poi
		shld	srcend
		call	copy_m	; umkopieren
		pop	h
		pop	psw
		push	h
inslop: 	mvi	m,00h		; NOP einfuegen
		inx	h
		dcr	a
		jnz	inslop
		pop	h
		jmp	datset	; Daten-Mode
;
; DELETE	loescht RAM-Zellen und rueckt RAM-Inhalt nach
;		bis RAM-Ende, keine Adressanpassung
;
fctdel: 	lda	adres		; Anzahl holen
		mov	e,a
		mvi	d,00h
		lhld	srcbeg
		shld	dstbeg
		push	h
		dad	d		; Startadresse
		shld	srcbeg
		lxi	h,buff_poi
		shld	srcend
		call	copy_m	; umkopieren
		pop	h
		jmp	datset	; Daten-Mode
;
; RAM-Bereich verschieben
;	
fctcpy: 	lhld	adres		; dritter Parameter
		shld	dstbeg
		call	copy_m	; kopieren
		lhld	dstbeg	; Zieladresse holen
		jmp	datset	; Daten-Mode
;	
copy_m: 	lhld	srcend	; COPY Speicherbereich verschieben
		xchg			; (DE) Ende Quellbereich
		lhld	dstbeg	; (HL) Anfang Zielbereich
		call	maxcom	; Vergleich (DE) mit (HL)
		mov	b,h
		mov	c,l		; (BC) Anfang Zielbereich
		lhld	srcbeg	; (HL) Anfang Quellbereich
		jnc	copyup
		mov	d,b
		mov	e,c		; (DE) Anfang Zielbereich
		call	maxcom	; vergleich
		push	h
		lhld	srcend
		xchg			; (DE) Ende Quellbereich
		pop	h		; (HL) Anfang Quellbereich
		jnc	copyup
		push	d
		push	h
		mov	a,l		; DSTEND berechnen
		cma
		mov	l,a
		mov	a,h
		cma	
		mov	h,a
		xchg			; (DE) kompl. Anfang Quellbereich
		lhld	srcend
		dad	d
		xchg			; (DE) Differenz SRCEND - SRCBEG
		lhld	dstbeg
		dad	d
		inx	h		; (HL) Ende Zielbereich
		mov	b,h
		mov	c,l
		pop	d
		pop	h
		;
copydw: 	mov	a,m		; abwrts kopieren
		stax	b
		call	comp
		rz			; fertig, ende
		dcx	h
		dcx	b
		jmp	copydw
					;	
copyup: 	mov	a,m		; aufwrts kopieren
		stax	b
		call	comp
		rz			; fertig, ende
		inx	h
		inx	b
		jmp	copyup
					;	
comp:  	mov	a,l		; Vergleich
		cmp	e
		rnz			; ungleich, ende
		mov	a,h
		cmp	d		; gleich (Z)=1
		ret	
;
maxcom: 	mov	a,h		; Vergleich
		cmp	d
		rc			; (DE) > (HL), ende
		rnz
		mov	a,l
		cmp	e
		ret			; (DE) > (HL), (CY)=1
;
; RAM mit Konstanten fuellen
;
fctfil: 	lda	adres		; Konstante holen
		sta	dstbeg
		call	fill_m	; Funktion ausfuehren
		lhld	srcbeg	; Startadresse holen
		jmp	datset	; Daten-Mode einstellen
;	
; FILL_m	fuellt RAM mit Konstante auf
;		SRCBEG = Anfangsadresse
;		SRCEND = Endadresse
;		DSTBEG = Konstante
;			
fill_m: 	lhld	srcend	; 
		xchg			; Anfang
		lhld	srcbeg	; Ende
		lda	dstbeg	; Konstante in DSTBEG
		mov	b,a		; Konstante
fillop: 	mov	m,b
		call	comp		; Ende ?
		inx	h
		rz
		jmp	fillop
;
; MAXRAM_m
;		ermittelt die hoechste RAM-Adresse (ab 8000h)
;		uebergabe in
;		(A) high, (B) low
;
maxram_m:	lxi	h,cpybuf	; Speichertest
		lxi	d,00fdh	; Grosse Speicherkarte
		mvi	b,49h		; Konstante
maxlop:	dad	d
		mov	c,m		; RAM-Inhalt retten
		mov	m,b		; Konstante in RAM
		mov	a,m		; Prueflesen
		mov	m,c		; alten Inhalt zurueckschreiben
		cmp	b		; Einschreiben erfolgreich ?
		jz	maxlop	; ja, weiter
		mov	a,h
		dcr	a		; MAX-Adr. high
		mvi	b,0ffh	; MAX-Adr. low
		ret			;
;
; Freiraum von 07E9 bis 07FF
;
;********************************************************************
;
; Echzeituhr mit MSM 5832
;
; Adresse   Bedeutung	Gueltigkeit Datenbit 0-3
;
; 3 2 1 0
; 0 0 0 0	= Sekunden-Einer	0 - 9
; 0 0 0 1	= Sekunden-Zehner	0 - 5
; 0 0 1 0	= Minuten-Einer	0 - 9
; 0 0 1 1	= Minuten-Zehner	0 - 5
; 0 1 0 0	= Stunden-Einer	0 - 9
; 0 1 0 1	= Stunden-Zehner	0 - 1 / 0 - 2 im 24Std-Modus
; 0 1 1 0	= Wochentag		0 - 6
; 0 1 1 1	= Tag-Einer		0 - 9
; 1 0 0 0	= Tag-Zehner	0 - 3 (D2 = 1 fuer Februar mit 29Tagen
;						/sonst 28Tage)
; 1 0 0 1	= Monat-Einer	0 - 9
; 1 0 1 0	= Monat-Zehner	0 - 1
; 1 0 1 1	= Jahr-Einer	0 - 9
; 1 1 0 0	= Jahr-Zehner	0 - 9
; 	
; Uhrzeit stellen (HL)Std,min, (BC)Wochentag, (DE)Monat,Jahr
;
	.org	0800h

;	
fctwur:	lhld	adres		; letzte Eingabe (Monat/Jahr)
		xchg	
		lhld	srcend	; vorl. Eingabe (Wochen-/Kalendertag)
		mov	c,l
		mov	b,h
		lhld	srcbeg	; erste Eingabe (Std, Min.)
		call	writu_m	; Zeit einschreiben
		jmp	fctrur
;	
writu_m:	push	d		; Mon./Jahr
		push	b		; W-K.Tag retten
		mov	a,h
		ori	80h		; 24 Std. Format
		mov	h,l		;
		mov	l,a		; (HL) Min./Std.
		mvi	c,08h		; Adresse Sekunden-Einer
		mov	a,c		;
		out	uhr_ctl		; Uhr anhalten
		call	dely1		; 1ms warten
		push	h		; Min./STd. retten
		mvi	l,00h		; Sekunden loeschen
		call	uhr_l		; (L) an Uhr
		pop	h
		call	uhr_hl	; (HL) an Uhr
		pop	d
		xchg
		mov	a,h
		ani	07h
		dcr	a		; Wochentagformat 0...6
		call	wr_5832	; Wochentag an MSM5832
		call	uhr_l		; Kalendertag an Uhr
		pop	d
		xchg
		call	uhr_hl	; (HL) an Uhr
		mvi	a,0f4h	; Maske 1111 0100 Hold zuruecksetzen
		out	uhr_ctl	; Uhrenbaustein wieder freigeben
		ret	
;
;
;	
uhr_hl:	mov	a,h		; (HL) an Uhrenbaustein
		call	uhr_ni
uhr_l:	mov	a,l		; (L) an Uhrenbaustein
		call	uhr_ni
		ret
;
; Daten in Nibble aufspalten und an Uhrenbaustein senden
;	
uhr_ni:	push	psw
		ani	0fh
		call	wr_5832	; unteres Nibble an Uhrenbaustein
		pop	psw
		rrc
		rrc
		rrc
		rrc
		ani	0fh
		call	wr_5832	; oberes Nibble an Uhrenbaustein
		ret
;
; wr_5832 Daten in MSM5832 schreiben
;	
wr_5832:	out	uhr_dat	; Daten an Uhrenbaustein
		mov	a,c		;
		out	uhr_ctl
		ori	02h		; 0000 0010 WR Signal takten
		out	uhr_ctl
		ani	0fdh		; 1111 1101 
		out	uhr_ctl
		mvi	a,10h		; naechste Registeradresse
		add	c
		mov	c,a
		ret
;
;
; Uhrzeit lesen BCD(HL)Adresse
;
;
fctrur: 	call	ezudsp_m	; EZU auslesen
		jmp fctrur		; Schleife
					;
readur_m:	lxi	h,buff_uhr	; BCD-Uhrbuffer
		push	h		;
		xchg			;
		mvi	c,0ch		; 0000 1100 Hold + /RD 
		mvi	a,08h		; 0000 1000 Bit 3 = HOLD 
		out	uhr_ctl	; Uhrenbaustein anhalten
		call	dely1		; 1ms warten
rduhr_l:	mov	a,c		;
		out	uhr_ctl		; /RD Uhr gesetzt
		push	h		; ein bischen
		pop	h		; warten
		in	uhr_dat		; Uhr lesen
		stax	d		; Zeichen in BCD-Buffer
		inx	d		;
		mvi	a,10h		; naechste Adresse einstellen
		add	c		;
		mov	c,a		;
		cpi	0dch		; alle Register gelesen
		jnz	rduhr_l	; nein -> naechsten Wert holen
		pop	h		;
		call	rdusort	;
		mvi	a,0f4h	; 11110100 = Hold + /RD 
		out	uhr_ctl	; zuruecksetzen
		ret
;
; RDUSORT = Uhrbuffer berichtigen/sortieren
;
; die im BCD-Buffer enthaltenen Daten 
; von den Adressinformationen (Hi-nibble) befreien
;	
rdusort: 	push	h		; BCD-Uhrbuffer
		lxi	b,0006h	; Offset Wochentag
		dad	b		;  
		mov	b,m		; Wochentag retten
		pop	h		;
		push h		; HL wieder auf Anfang BCD-Buffer
		xchg			; und DE
		pop	h		; HL = Schreibzeiger
		push	h		; DE = Lesezeiger
		call	bit4		; Sek-Einer holen Maskieren und wieder ablegen
		call	bit3		; Sek-Zehner
		call	bit4		; Minuten-Einer
		call	bit3		; Minuten-Zehner
		call	bit4		; Stunden-Einer
		ldax	d		; Stunden-Zehner
		ani	03h		; mit 0000 0011 maskieren
		mov	m,a		;
		inx	d		;
		inx	d		;
		inx	h		;
		call	bit4		; Tag-Einer
		ldax d		; Tag-Zehner
		ani	03h		; mit 0000 0011 maskieren
		mov	m,a		;
		inx	d		;
		inx	h		;
		inx	h		;
		call	bit4		; Wochentag			
		ldax	d		; Dummy
		ani	01h		; mit 0000 0001 maskieren
		mov	m,a		;
		inx	d		;
		inx	h		;
		call	bit4		; Monat-Einer
		call	bit4		; Monat-Zehner
		dcx	d		; Lesezeiger wieder Monat-Zehner 
		mvi	c,04h
rdu_bl:	ldax	d		; Monat-Zehner holen
		mov	m,a
		dcx	d		;
		dcx	h		;
		dcr	c		;
		jnz	rdu_bl	;
		mov	a,b
		ani	07h
		inr	a
		mvi	m,00h
		dcx	h
		mov	m,a
		pop	h
		ret	
;
; BCD-Zahlen oberstes Nibble(Adressdaten) ausblenden 
;	
bit3:  	ldax	d		; Maske 0000 0111 
		ani	07h
		jmp	bitend
bit4:		ldax	d		; Maske 0000 1111
		ani	0fh
bitend:	mov	m,a		; Byte wieder sichern
		inx	d		;
		inx	h		; naechste Adresse
		ret
;
;
;-------------------------------------------------------------------------
;
; EZUDSP_m	
;	Uhrzeit in Siebensegmentanzeige schreiben
;
;
ezudsp_m:	call readur_m	; Uhrzeit lesen
		lxi	d,dig1
		call	ezusub	; Sekunden
		inx	d		; DIG ueberspringen
		call	ezusub	; Minuten
		call	ezusub	; Stunden
		lda	dig6		;
		ori	80h		; Dezimalpunkt einblenden
		sta	dig6
		call	dely100	; SSG Bufferanzeigen
		xra	a		;
		sta	dspbuf	; LED's dunkel
		ret
					;
ezusub:	mov	b,m		; erstes BCD-Digit
		inx	h
		mov	a,m		; zweites BCD-Digit
		inx	h
		rlc	
		rlc	
		rlc	
		rlc	
		ora	b		; verknuepfen
		push	h
		call	cnvbyt	; in SSG-Code umsetzen
		pop	h
		ret
;
; Uhrzeit im ASCII-Buffer schreiben
; ! wird von rechts nach links eingetragen
;	
ti_m:		call	readur_m	; Uhr auslesen
		xchg
		dcx	d
		lxi	h,2fffh	; Zeiger auf Ende ASCII-Uhrbuffer
		mvi	m,00h		; 00h eintragen fuer String-Ende
		dcx	h
		mvi	m,68h		; ASCII "h" speichern
		call	h2bcd		; Sekunden in ASCII Buffer 
		dcx	h
		mvi	m,3ah		; ASCII ":" (Trennzeichen)
		call h2bcd		; Minuten in ASCII Buffer
		dcx	h
		mvi	m,3ah		; ASCII ":" 
		call	h2bcd		; Stunden in ASCII Buffer
		dcx	h
		mvi	m,3bh		; ASCII ";"
		lxi	d,uhrmonz
		call	h2bcd		; Jahr in ASCII Buffer
		dcx	h
		mvi	m,2eh		; ASCII "."
		lxi	d,uhrwtu
		call	h2bcd		; Monat in ASCII Buffer
		dcx	h
		mvi	m,2eh		; ASCII "."
		lxi	d,uhrstdz
		call	h2bcd		; Tag in ASCII Buffer
		dcx	h
		mvi	m,2ch		; ASCII ","
		dcx	h
		push	h		;
		lxi	h,wt_tab	; Tabelle fuer Wochentag 
		lda	uhrwtl	; Wochentag holen
		ani	07h		; mit Tabelle umsetzen
		rlc			;
		mov	e,a		;
		mvi	d,00h		;
		dad	d		;
		mov	e,m		; Wochentag-Symbol low
		inx	h		;
		mov	d,m		; Wochentag-Symbol high
		pop	h		;
		mov	m,e		;
		dcx	h		;
		mov	m,d		;
		ret			;
;
; h2bcd = hole 2 BCD-Werte aus Buffer
;	
h2bcd:	call	hbcd		; 
;
hbcd:		dcx	h		;
		inx	d
		ldax	d
		adi	30h		; Gueltigkeit pruefen
		cpi	3ah		; ASCII "9" + 1
		jnc	hbcd_er	; Fehler
		cpi	30h
		jnc	hbcde		; OK Wert zueruckgeben
hbcd_er:	mvi	a,2dh		; Wert ungueltig "-" 
					; zurueckgeben
hbcde:	mov	m,a		;
		ret			;
;
; Wochentage ..oMiDiMoDrFaSoS
; 1 = Montag, 7 = Sonntag 
;
;
wt_tab:	.db	00h		;                 
		.db	00h		;
		.db	6fh		; "o" - 
		.db	4dh		; "M" - Montag
		.db	69h		; "i"
		.db	44h		; "D"
		.db	69h		; "i"
		.db	4dh		; "M"
		.db	6fh		; "o"
		.db	44h		; "D"
		.db	72h		; "r"
		.db	46h		; "F"
		.db	61h		; "a"
		.db	53h		; "S"
		.db	6fh		; "o"
		.db	53h		; "S" - Sonntag
;
;
;
;====================================================================
;
; DSPACC_m	
;		Akku-Inhalt in Siebensegment-
;		Code und binaer in LED-Zeile darstellen
;
dspacc_m:	push	psw		;
		lxi	d,dig1
		call	cnvbyt	; SSG-Umsetzung
		pop	psw
		sta	dspbuf	; (ACC) in LED-Zeile
		lxi	h,0000h
		shld	dig4		; oberes Digit dunkel
		shld	dig6
dsplop:	call	press		; auf Taste warten
		jmp	dsplop
;
; DSPHL_m	Inhalt von H&L im Siebensegment-Code darsellen
;	
dsphl_m:	push	h		; retten 
		lxi	d,dig4	; rechte Adressanzeige
		mov	a,l
		call	cnvbyt
		pop	h
		mov	a,h
		call	cnvbyt
dspcte:	lxi	h,0000h	; unteres Digit dunkel
		shld dig1
		jmp dsplop
;
; DSPERR	Fehleranzeige "ErrX"
;				
;	
dsperr_m:	lxi	d,dig4	; rechte Adressanzeige
		call	cnvbyt
		lxi	h,dig7
		mvi	m,79h		; "E"
		dcx	h
		mvi	m,50h		; "r"
		dcx	h
		mvi	m,50h		; "r"
		jmp	dspcte
;
; Freiraum bis 09ffh
;********************************************************************
;
; LEDASC	Monitor fuer ASCII-Tastatur u. LED-Display
;
;	Die Eingabe kann Wahlweise ueber die an der 
;	ASCII-Tastatur angeflanschte HEX-Tastatur erfolgen
;	oder ueber die ASCII-Tasten
;	
;	DAT:	= Pfeil rechts
;	ADR:	= Pfeil links
;	REG:	= "r" oder "R"
;	NXT:	= Return
;	BST:	= Pfeil oben
;	RUN:	= "g" oder "G"
;	FCT:	= Funktionstasten
;	
	.org	0a00h
;	
ledasc:	call	csts		; auf loslassen warten
		jnz	ledasc
		call	init		; Grundeinstellung
		call	updhex
ledmo:	call	hexky2	; HEX-tasten abfragen
		call	ssgdsp	; SSG-Anzeige aktivieren
		call	cmdky2
		jmp	ledmo		; Schleife
		;
hexky2:	call	csts		; Taste aktiv
		rz			; nein, ende
		call	ci
		mov	b,a
		mvi	b,0ah
		cpi	82h		; Cursor rechts
		jz	hexcte
		mvi	b,0bh
		cpi	8bh		; Cursor down
		jz	hexcte
		mvi	b,0ch
		cpi	80h		; Cursor left
		jz	hexcte
		mvi	b,0dh
		cpi	2fh		; Slash
		jz	hexcte
		mvi	b,0eh
		cpi	89h		; Cursor Up
		jz	hexcte
		mvi	b,0fh
		cpi	8fh		; Cursor Home
		jz	hexcte
		call	hxtest_v
		rz
		;
hexcte:	mov	c,b
		mov	a,c
		call	shift		; Eingabe nachschieben
		call	convt		; SSG-Umsetzung
hexlop:	call ssgdsp
		call csts		; auf loslassen warten
		jnz hexlop
		ret
;
cmdky2:	lxi	h,cmlop	; CTE-Sprungzuiel
		push	h
		in	0ch		; FCT-Spalte abfragen
		lxi	h,ciflg
		xra	m
		cpi	20h
		jz	fct
;
		call	csts		; Taste aktiv
		pop	h
		rz			; nein, ende
		push	h
		call	ci		; ja ASCII-Code holen
		mov	b,a
		lda	modflg	; FCT-Modeaktiv ?
		ani	08h
		mov	a,b
		jz	cmdcte		; nein, weiter
		cpi	0dh		; RETURN
		jz	brnch
		cpi	20h		; Blank
		jz	brnch
		cpi	3ch		; Pfeil links
		jz	brnch
		cpi	2dh		; Minus
		jz	brnch
		cpi	72h		; "r"
		jz	brnch
		cpi	52h		; "R"
		jz	brnch
		cpi	2bh		; "+"
		jz	brnch
cmdcte:	cpi	3eh		; Pfeil rechts
		jz	dat
		cpi	2ah		; "*"
		jz	dat
		cpi	3ch		; Pfeil links
		jz	adr
		cpi	2dh		; Minus
		jz	adr
		cpi	72h		; "r"
		jz	reg
		cpi	52h		; "R"
		jz	reg
		cpi	2bh		; "+"
		jz	reg
		cpi	0dh		; RETURN
		jz	nxt
		cpi	20h		; Blank
		jz	nxt
		cpi	5eh		; Pfeil hoch
		jz	bst
		cpi	2eh		; "."
		jz	bst
		cpi	67h		; "g"
		jz	run
		cpi	47h		; "G"
		jz	run
		cpi	3fh		; "?"
		jz	run
		pop	h
		ret
					;
cmlop:	call	ssgdsp
		call	csts
		jnz	cmlop		;
		ret
;
; Freiraum von 0AF7 bis 0AFF
;
;********************************************************************
; 
; Ansteuerung von Peripheriegerten
;	
; Unterprogramme
;	- CSINIT_m	Initialisierung	CAS-Interface
;	- LOINIT_m		"	PRT-Interface
;	- V24ini_m		"	V24 Schnittstelle
;	- PO		Casettenausgabe
;	- RI		Casetteneingabe
;	- V24Out	Ausgabe V24
;	- V24in		Eingabe V24
;	- CPUOT		SOD-Ausgabe
;	- CPUIN		SID-Eingabe
;
	.org	0b00h
;	
csinit_m:	push	d		; Casetteninterface vorbereiten
		push	h
		lda	casmod	; Modus laden
		mov	b,a
		mvi	a,0c2h	;
		lhld	casbdr	; Baudrate
		jmp	initct
					;
lonit_m:	push	d		; Serieller Drucker vorbereiten
		push 	h
		mvi	a,8ah		; 8255 initialisieren
		out	par_ctl	; PA, PClo = Out
		mvi	a,01h		; PB, PChi = IN
		out	par_pc	; Strobe auf High
		lxi	h,buff_txt	; Themodrucker vorbereiten
		shld	buff_poi	;
		xra	a		;
		sta	buff_num	; Zeichenzaehler loeschen
		mvi	a,01h		;
		out	0bah		; Centronis vorbereiten
		lda	prtmod	;
		mov	b,a		;
		mvi	a,0a0h	; Moderegister
		lhld	prtbdr	;
		jmp	initct	;
;
; Sprungtabelle fuer V24 Routinen
; werden ueber PCHL angesprungen
;
; 06.06.2017	Timer als BCD-Zaehler
;			ACIA Betriebsart auf Asynchron umgestellt
;	
v24ini_m:	jmp	initv24	; --> INIT V24
		jmp	lo7		; V24 Ausgabe
		jmp	ri_m		; V24 Eingabe
;	
initv24:	push	d		; INIT V24-Schnittstelle
		push	h
		lda	v24mod	;
		mov	b,a
		mvi	a,58h
		lhld	v24bdr	; Baudrate laden
initct:	out	cs_mode	; Modus ??
		mvi	a,03h
		out	acia_ctl	; Reset ACIA
		mov	a,b
		out	acia_ctl	; Datenuebertragung (bit,1Stop,ohne Paritaet)
		mvi	a,37h		; als BCD-Zaehler
		out	timer_ctl	; Timer_0 Mode 3, erst unt., dann ober.Byte
		call	bd_hol	; Bd-Werte ermitteln Rueckgabe (HL)
		mov	a,l		; Startwerte fuer Baudrate laden
		out	timer_0	; Teilfaktor low
		mov	a,h
		out	timer_0	; Teilfaktor high
		pop	h
		pop	d
		ret
;
; BD_HOL (BAUDRT)	
;		ermittelt aus der Baudrate und
;		CPU Takt (FQuarz) den Timer Teilfaktor
;		Siehe Tabelle
;					
bd_hol: 	mov	a,h		; Timerwerte fuer Baudrate setzen
		rlc
		mvi	a,00h
		adc	h
		ani	07h		; Versatz
		rlc	
		mov	e,a
		mvi	d,00h		; Zeiger innerhalb der Tabell
		lxi	h,bd_t3	; Tabelle hohe Frequenz auswaehlen
		lda	fquarz	; (06h) <-- aus Adresse 0006h CPU-Takt
		cpi	07h
		jnc	bd_end	; CPU-Takt > 7MHz
		lxi	h,bd_t2
		cpi	05h
		jnc	bd_end	; CPU-Takt > 5MHz < 7 Mhz
		lxi	h,bd_t1	; CPU-Takt < 5 MHz
bd_end:	dad	d
		mov	e,m		; Werte aus Tabelle holen
		inx	h
		mov	d,m
		xchg
		ret			; (HL)Timerwerte fuer Baudrate
;
; Baudratentabell als Binaerwert
;
; 06.06.2017 Tabelle fuer den Asynchronbetrieb neu aufgebaut
; (urspruenglich nur Synchronbetrieb)
;
; Teilfaktor fuer Timer = CPU-Frequenz /2 = Systemtakt / 4800bd / 16 = 40
;                                                                  I
;                                                                  UART Teiler
; Wert als BCD, da Timer auf BCD Zaehler eingestellt wird
;               sonst Umrechnung nach Binaer notwendig
;
; 9600Bd = 20h lowByte
; 4800Bd = 40h .
; 2400Bd = 80h .
; 1200Bd = 60h lowByte
;	   01h highByte
;
; Quarz < 5Mhz hier 4,9152 Mhz
;
bd_t1:	.db	32h		; 4800 Bd* Standard
		.db	00h		; 
		.db	96h		;  110Bd
		.db	13h		;
		.db	28h		; 1200Bd
		.db	12h		;
		.db	05h		;  300Bd
		.db	05h		;
		.db	64h		; 2400Bd
		.db	00h		;
		.db	32h		; 4800Bd
		.db	00h		;
		.db	56h		;  600Bd
		.db	02h		; 
		.db	16h		; 9600Bd
		.db	00h
;
; Quarz > 5 MHz < 7MHz , hier 6,144MHz	
;
bd_t2:	.db	40h		; 4800Bd
		.db	00h
		.db	45h		;  110Bd
		.db	17h
		.db	60h		; 1200Bd
		.db	01h
		.db	40h		;  300Bd
		.db	06h
		.db	80h		; 2400Bd
		.db	00h
		.db	40h		; 4800Bd
		.db	00h
		.db	20h		;  600Bd
		.db	03h
		.db	20h		; 9600Bd
		.db	00h 
;
; Quarz > 7Mhz 9,8304MHz (8085A2 bis 10Mhz)
;
bd_t3:	.db	64h		; 4800Bd	
		.db	00h
		.db	93h		;  110Bd
		.db	27h
		.db	56h		; 1200Bd
		.db	02h
		.db	24h		;  300Bd
		.db	10h
		.db	28h		; 2400Bd
		.db	01h
		.db	64h		; 4800Bd
		.db	00h
		.db	12h		;  600Bd
		.db	05h
		.db	32h		; 9600Bd
		.db	00h
;	
;====================================================================
; 
; Casettenausgabe
; 
fctcot:	lhld	adres		; letzten Parameter holen
		shld	srcend	; und ablegen
;
;
casot_m:	lda	srcbeg	; Abfrage wg. Abgleich
		cpi	0adh		; ad=Adjust
		jz	adjust	; ADJUST
		call	csinit	; Cas-Ausgabe	
		lxi 	b,0258h	; = 600d x 8,3ms
outlop:	xra	a
		call	casot1	; Casot1 Byte ausgeben
		dcx	b
		cmp	c
		jnz	outlop
		cmp	b
		jnz	outlop
		mvi	a,0d3h	; Startzeichen
		call	casot2	; CASOT2 3mal ausgeben
		call	casot1	; CASOT1
		xra	a
		call	casot1	; CASOT1 "00" als Abschlu
		lhld	srcend	; Parameter holen
		inx	h
		xchg
		lhld	srcbeg
output:	mov	c,m		; OUTPUT Zeichen holen
		call	po_m		; ausgeben
		inx	h
		mov	a,l		; Endabfrage
		cmp	e
		jnz	output	; weiter
		mov	a,h
		cmp	d
		jnz	output	; weiter
		xra	a
		call	casot2	; CASOT2
		call	casot2	; CASOT2
		call	lonit_m
		ret
;
; Casetteneingabe
;
; Programm mit 5s Vorspann laden
;
fctcin:	lhld	adres		; letzten Parameter holen	
		shld	srcend	; und ablegen
		call	casin_m	;
		rnz			; OK Return
		jmp	dsperr	; Fehler
			
casin_m:	call	csinit	;
		mvi	b,64h		; 100d Bytes erfassen
inlopo:	mvi	c,0ah		; 10d Bytes vorlauf
inlpo:	call	ri_m
		cpi	00h
		jz	inlo		; "00" erkannt
		dcr	b
		jnz	inlopo
		jmp	invert
		;
inlo:		dcr	c
		jnz	inlpo
		jmp	inlop2	; auf Startmaske "D3" warten
					;
invert:	mvi	a,0c3h
		out	cs_mode
		mvi	b,64h		; 100Bytes erfassen
inlop1:	mvi	c,0ah		; 10Bytes vorlauf
inlp1:	call	ri_m
		cpi	00h
		jz	inl1		; "00" erkannt
		dcr	b
		jnz	inlop1
		mvi	a,00h		; Vorspannfehler	
		jmp	inend		; Fehler: (Z)=1
inl1:		dcr	c
		jnz	inlp1		;
inlop2:	mvi	c,03h		; 3 StartBytes
inlop3:	call	ri_m
		cpi	0d3h
		jnz	inlop2	; "D3" nicht erkannt
		dcr	c
		jnz	inlop3	;
		jmp	inlop4	; 3x "D3" erkannt
		;
		mvi	a,01h		; Startzeichenfehler
		jmp	inend		; Fehler (Z)=1
		;
inlop4:	call	ri_m		;
		cpi	00h
		jz	input		; "00" erkannt
		xra	a
		mvi	a,02h		; Synchron Fehler
		jmp	inend		; Fehler (Z)=1
					;
input:	lhld	srcend
		inx	h
		xchg
		lhld	srcbeg
inlp:		call	ri_m		; Zeichen einlesen
		rz			; ENDE bei Datenverlust
		mov	m,a		; OK, Daten ins RAM ablegen
		inx	h
		mov	a,l		; Endabfrage
		cmp	e
		jnz	inlp		; nein, weiter
		mov	a,h
		cmp	d
		jnz	inlp		; nein, weiter
		xra	a
		inr	a		; OK (Z)=0
inend:	push	psw
		call	lonit_m
		pop	psw
		ret			;
;
;====================================================================
;
; Casetteninterface abgleich
;
; 		Testpunkt TP0 an Masse legen, TP1 mit
;		Timereingang G0 verbinden und Abgleichroutine aufrufen:
; 			ingabe "o ad", RETURN
; 		("o" = Casettenausgabe, "ad" fr Adjust)
;		Sollwerte:	4,000 Mhz 3750d
;				4,194 MHz	3690d
;				6,000 MHz	3125d
;				6,144 MHz	3080d
;				8,000 MHz	2500d
;
adjust:	mvi	a,31h		; Mod 0, BCD
		out	timer_ctl	; als BCD Zaehler
		mvi	a,00h
		out	timer_0
		mvi	a,50h		; Anfangswert 5000d 
		out	timer_0
		mvi	a,01h		; XOR triggern
		out	cs_mode		
		xra	a		; 
		out	cs_mode
		call	dely1	; 1ms warten
		mvi	a,01h		; Timer lesen
		out	timer_ctl	;
		in	timer_0	; low Byte
		mov	l,a
		in	timer_0	; high Byte
		mov	h,a
		xchg
		lhld	update	; Cursor Update
		push	h
		xchg
		call	adrot_v	; Zaehlerstand anzeigen
		pop	h
		shld	update
		call	onesec	; beruhigen
		jmp	adjust
;	
; CASOT1/2 gibt 1 bzw. 2Bytes ueber das Magnetbandinterface aus
;
casot2:	call	casot1
casot1:	push	b
		mov	c,a
		push	psw
		call	po_m		; Zeichen ausgeben
		pop	psw
		pop	b
		ret
;
;====================================================================
;
; RI_m 	liest ein Byte ueber das Magnetbandinterface ein (A)
;	
ri_m:		push	b		;
		mvi	c,0ch		; max 12ms warten
rilop: 	in	acia_ctl	; warten auf Daten
		ani	01h		; Empfangsregister voll
		jnz	ricte
		call	dely1
		dcr	c
		jnz	rilop		; maximal 12ms (C)
		mvi	a,03h		; Zeitueberschreitung 
		pop	b
		ret			; Fehler (Z)=1
ricte:	in	acia_dat	; Daten einlesen
		pop	b
		ret			; OK Zeichen in (A)
;
; PO_m 	gibt ein Zeichen ueber das Magnetbandinterface aus (C)
;	
po_m:		mov	b,c		; Daten (C) an kassetteninterface
		mvi	c,0ch		; Wartezeit 12ms
polop:	in	acia_ctl	; warten bis
		ani	02h		; Senderegister leer
		jnz	pocte
		call	dely1		; maximal 12ms (C)
		dcr	c
		jnz	polop
		mvi	a,04h		; Gegenstelle nicht bereit
		ret			; Fehler (Z)=1
pocte:	mov	a,b		; Daten ausgeben
		out	acia_dat
		ret			; OK
;
; Daten Ausgabe je nach gewaehlter Schnittstelle
; (2FC8h) Bit 7 6 5 4 bestimmen den kanal
;
;	Bit7	= V24
;	Bit6	= Centronis
;	Bit5	= Paralelle Ausgabe
;	Bit4	= Thermodrucker;
; 
lo_m:		lda	ioflg		; Daten (C) an Drucker mit 4800Bd
		ani	0f0h		; Bit 4-7 maskieren
		rlc
		jc	lo7		; LO7 = V24
		rlc
		jc	lo6		; LO6 = Centronics
		rlc
		jc	lo5		; LO5 = Paralele Schnittstelle
		rlc
		jc	lo4		; LO4 = Thermodrucker
		xra	a
		mvi	a,05h		; I/O Byte undefiniert
		ret
;
timout	.equ	2000h		; ca 8000ms = 8s
;	
lo7:		push	b		; V24 schreiben
		lxi	b,timout	;
lo7lop:	in acia_ctl
		ani	02h		; Statusbit maskieren
		jnz	lo7cts	; Senderegister leer	
		call	dely1		; Wartezeit max 2sek (BC)
		dcx	b
		xra	a
		cmp	b
		jnz	lo7lop
		cmp	c
		jnz	lo7lop
		pop	b
		mvi	a,07h		; Zeitueberschreitung 
		ret			; Gegenstelle nicht bereit
;
lo7cts:	in	acia_ctl
		ani	08h
		jnz	lo7cts	; warten CTS Eingang
lo_cte:	lxi	b,0000h	; Flags loeschen	
		push	b
		pop	psw
		pop	b
		mov	a,c
		out	acia_dat	; Daten ausgeben
		ret
;
;====================================================================
;
; Daten an Centronis Schnittstelle ausgeben
; Port CA,BA
;
lo6:		push	b
		lxi	b,timout
lo6lop:	in	0cah		; Centronics BUSY
		ani	40h
		jnz	lo6cte
		call	dely1
		dcx	b
		xra	a		; Endabfrage
		cmp	b
		jnz	lo6lop
		cmp	c
		jnz	lo6lop
		pop	b
		mvi	a,06h		; Gegenstelle nicht bereit
		ret			; FEhler (Z)=1
lo6cte:	pop	b
		mov	a,c
		out	0cah		; Daten an Centronics
		out	0bah		; Strobe ausgeben
		ret
;
; Ausgabe an Paralelle Schnittstelle 8255
;	
lo5:		push	b
		lxi	b,timout	;
lo5lop:	in	par_pc	; BUSY Abfrage
		ani	80h
		jz	lo5cte	;
		call	dely1
		dcx	b
		xra	a		; Endabfrage
		cmp	b
		jnz	lo5lop
		cmp	c
		jnz	lo5lop
		pop	b
		mvi	a,08h		; Gegenstelle nicht bereit
		ret			; Fehler (Z)=1
;
lo5cte:	pop	b
		mov	a,c
		out	par_pa	; Daten ausgeben
		xra	a
		out	par_pc	; Strobe 	
		call	lo5end	; Wartezeit
		inr	a
		out	par_pc
lo5end:	ret			;
;
; Ausgabe auf Thermodrucker
;
lo4:		mov	a,c		;
		cpi	08h		; BS?
		jz	thddel
		cpi	0ch		; FF?
		jz	thdff
		cpi	0dh		; CR?
		jz	thdcr
		cpi	7fh		; DEL?
		jz	thddel
		cpi	20h		; Code im zulaessigen Bereich ?
		jc	lo4ret
		cpi	0a0h		; LF?
		jc	lo4cte
lo4ret:	nop
		mvi	a,09h		; unzulaessiger Code
		ret			; Fehler (Z)=1
;	
lo4cte:	lhld	buff_poi	;
		mov	m,c		; Zeichen in Textbuffer
		inx	h		; Bufferzeiger erhoehen
		shld	buff_poi
		lda	buff_num	; Zeichenzaehler erhoehen
		inr	a
		sta	buff_num	;
		cpi	14h		; Buffer voll ?
		rnz			; OK
		;	
thdcr:	xra	a		; Zeichenzaehler loeschen
		sta	buff_num
		lxi	h,buff_txt	; Zeiger auf Textanfang
		shld	buff_poi
		call	thermo	; Sprung nach Adr 2014h 
					; Thermodrucker ?
thdff:	mvi	a,14h		; Buffer loeschen
		lxi	h,buff_txt	;
thdffl:	mvi	m,20h
		inx	h
		dcr	a
		jnz	thdffl	; fertig ?
		inr	a
		ret
		;
thddel:	lda	buff_num	; Zeichen loeschen
		dcr	a
		rm			; Buffer war leer
		sta	buff_num
		lhld	buff_poi
		dcx	h
		mvi	m,20h		; mit "Blank" ueberschreiben
		shld	buff_poi
		ret
;
; Serielle Schnittstelle der CPU ansteuern
; SID Geraet A mit SOD Geraet B
; SOD Geraet A mit SID Gereat B
; und GND verbinden
;
fctsod:	lhld	adres		; SOD Ausgabe
		inx	h
		xchg			; (DE) Endadress
		lhld	srcbeg	; (HL) Startadresse
sodlop:	mov	c,m		; Zeichen holen
		call	cpuot		; und ausgeben
		inx	h
		mov	a,l		; Endabfrage
		cmp	e
		jnz	sodlop	; weiter
		mov	a,h
		cmp	d
		jnz	sodlop	; weiter
		ret
;		
fctsid:	lhld	adres	; SID eingabe
		inx	h
		xchg			; (DE) Endadresse
		lhld	srcbeg	; (HL) Staradresse
		mvi	a,0c0h	; SOD auf High
		sim			;
sidlop:	call	cpuin		; Zeichen holen
		mov	m,a		; ablegen
		inx	h
		mov	a,l		; Endabfrage
		cmp	e
		jnz	sidlop	; weiter
		mov	a,h
		cmp	d
		jnz	sidlop	; Weiter
		ret
;
; CPUOT	Ausgabe (C) ueber die CPU-Schnittstelle SOD
;
cpuot:	rim			; Ready abfragen
		ana	a
		jp	cpuot		; SID auf LOW, warten
		mov	a,c		; Zeichen holen
		call	stabit	; Startbit ausgeben
		mvi	c,08h		; Bitzaehler
otlop:	rrc			; LSB zuerst ausgeben
		ana	a
		jm	high		; Bit ist High
low:		call	stabit	; LOW-Bit ausgeben	
		jmp outcte	;
			
high:		call	stobit	; High-Bit ausgeben
outcte:	dcr	c
		jnz	otlop		;
		call	stobit	; Stopbit ausgeben
		ret			;
stabit:	push	psw
		mvi	a,0c0h	; SOD auf High
		jmp	bitcte	;
stobit:	push	psw
		mvi	a,40h		; SOD auf Low
bitcte:	sim
		call	dely2		; Wartezeit
		pop	psw
		ret
;
; CPUIN 	Daten ueber die CPU-Schnittstelle entgegennehmen SID
;
cpuin:	rim			; SID abfragen
		ana	a
		jp	cpuin		; SID auf low, warten
		push	d
		mvi	d,08h		; Bitzaehler
		mvi	e,00h		; Schieberegister
		call	dely1		; halbe Bit-Dauer warten
inlop:	call	dely2		; mitte Datenbit
		rim
		ani	80h		; MSB maskieren
		ora	e		; verknpfen
		rrc
		mov	e,a		; ins Schieberegister
		dcr	d
		jnz	inlop		;
		rlc
		push	psw
		call	dely2		; mitte Stopbit
		pop	psw
		pop	d
		ret
		;
dely2:	call	dely1		; 2ms warten
		call	dely1
		ret
;
; Ansteuerung Thermodrucker
; nur moeglich wenn kein Floppy-Controler eingebaut ist
; bzw. mit Umschalter der Adressbereich 2000h umgestellt wird
;
fcthex:	lhld	adres		; HEX-Dump
		xchg			; (DE) Endadress
		lhld	srcbeg	; (HL) Startadress
		mvi	a,40h		; Portadresse FDC
		out	fdc_sel
		call	fdck
		xra	a
		out	fdc_sel
		inr	a
		ret			;
fctasc:	lhld	adres		; ASCII ausdruck
		shld	srcend	; Zeilenzahl
		lhld	srcbeg	; String Anfang
		mvi	a,40h
		out	fdc_sel
		call	fdcw
		xra	a
		out	fdc_sel
		inr	a
		ret
; frei
		rst	7
		rst	7
		rst	7
		rst	7
		rst	7
		rst	7
		rst	7
;
;====================================================================
;
; Consolenausgabe mit Filter fuer Sonderzeichen
;
; Anpassung des TEXASS-Textverarbeitungsprogramm
;
; Erweiterung des Mode-Flags
;
;	Bit7 6 5 4 3 2 1 0
;	   I I I I I I I I
;        I I I I I I I + - Daten-Mode
;        I I I I I I + - - Adress-Mode
;        I I I I I + - - - Register-Mode
;        I I I I + - - - - Funktionseingabe
;	   I I I + - - - - - ESC-0 wird von "1B" ESC gesetzt
;	   I I + - - - - - - ESC-1 zweiter Parameter
;	   I I + - - - - - - ESC-2 dritter Parameter
;	   + - - - - - - - - Einzelschrittbetrieb
;
co_m:		push	psw		; ESCAPE
		push	h
		push	d
		push	b
		mvi	a,0c3h	; Hilfszellen fuer Sprungziel
		sta	brkpt1	; JMP 1006h Videomonitor
		lxi	h,co_v	; bzw. je nach Sondertasten
		shld	hbufe		;
		lda	modflg
		mov	b,a
		ani	10h		; ESC-Flag abfrageb
		jnz	estest	; gesetzt, weiter
		mov	a,c		; ASCII
		cpi	1bh		; = ESC ?
		jz	escset	; ja, ESC-Flag setzen
		mov	a,b		;
		jmp	escend	;
					;	
estest:	mov	a,b		;
		ani	40h		; ESC-Flag 2 abfragen
		jz	escte1	; gesetzt, weiter
		mov	a,c		;
		sta	hbuf3		; CO-Byte ablegen
		mvi	c,16h		; = TSCPOS
		jmp	esprcs	;
;
escte1:	mov	a,b		;
		ani	20h		; ESC-Flag 1 abfragen
		jz	escte2	; geloescht, weiter
		mov	a,c
		sta	hbuf3		; CO-byte ablegen
		mov	a,b
		ori	40h		; ESC-Flag 2 setzen
		jmp	escend
;	
escte2:	mov	a,c		; ASCII
		cpi	16h		; TSCPOS?
		jnz	esprcs	; nein, weiter
		mov	a,b
		ori	20h		; ESC-Flag 1 setzen
		jmp	escend
;
esprcs:	mov	a,c		; ASCII
		cpi	20h		; kleinstes ASCII zeichen	
		jnc	escl		; ist ASCII, weiter
		cpi	1ah		; oberer Grenzwert
		jnc	esclr		; ueberschritten, weiter
		cpi	10h		; unterer Grenzwert
		jc	esclr		; unterschritten, weiter
		lxi	h,esclr	; CTE-Adresse
		push	h		; sichern
		lxi	h,cojt1	; 1.Adresse aus JMP Tabelle
		sui	10h		; ASCII-Wert - 10
		rlc			;  * 2
		mov	e,a		; 
		mvi	d,00h		; (DE) Versatz
		dad	d		; auf JMP-Tabelle
		mov	e,m		; neue Adresse aus JMP-Tabelle laden
		inx	h		;
		mov	d,m		;
		xchg			; (HL) Zieladresse
		pchl			; Sprung zu Zieladresse 
;	
escl:		lda	modflg
		ani	8fh
		jmp	escend
;	
esclr:	lda	modflg
		ani	8fh		; ESC-Flags loeschen
escnd:	lxi	h,escret	;  = RET
		shld	hbufe		; in Hilfszelle
escend:	sta modflg
		pop	b
		pop	d
		pop	h
		pop	psw
		jmp	brkpt1	; Sprung ueber Hilfszellen
					; 
;	
escset:	mov	a,b
		ori	10h
		jmp	escnd		;
;
; Sprungtabelle je nach ASCII-Code
;
; Labels als CO JMP TAB n vergeben
;             "  "   "  " z = Ziel 
;
	
cojt1:	.dw	clrvid_v	; ESC-P = Spung nach Clearvid 102Dh
;        	
cojt2:	.dw	tchome	; ESC-Q = Cursor Home
;	
cojt3:	.dw	crsaus_v  	; ESC-R = Cursor aus
;	
cojt4:	.dw	crsein_v	; ESC-S = Cursor Ein
;		
cojt5:	.dw	cojt5z	; ESC-T = Scroll up
;		
cojt6:	.dw	cojt6z	; ESC-U = Scroll down
		
cojt7:	.dw	cojt7z	; ESC-V = Cursor Position (LL/CC)
		
cojt8:	.dw	cojt8z	; ESC-W = Schirm ab Cursor loeschen
;		
cojt9:	.dw	cojt9z	; ESC-X = Zeile loeschen
;	
cojt10:	.dw	cojt10z	; ESC-Y = Zeile ab Cursor loeschen
;	
tchome:	lxi	h,home	;
		jmp	crspos_v	;
;	
cojt5z:	lxi	b,bcup	;
		lxi	d,deup
		lxi	h,hlup
uplop:	in	vid_dspen	; Ausgabe mit Strahlrueklauf
		rrc			; Synchronisieren
		jnc	uplop		; Blanking abwarten
		mov	a,m
		stax	b
		inx	b
		inx	h
		mov	a,e		; Endabfrage
		cmp	l
		jnz	uplop
		mov	a,d
		cmp	h
		jnz	uplop
		ret
;	
cojt6z:	lxi	b,bcdwn	;
		lxi	d,dedwn
		lxi	h,hldwn
dwnlop:	in	vid_dspen 	; Ausgabe mit Strahlrueklauf
		rrc			; Synchronisieren
		jnc	dwnlop	; Blanking abwarten
		mov	a,m
		stax	b
		dcx	b
		dcx	h
		mov	a,e		; Endabfrage
		cmp	l
		jnz	dwnlop
		mov	a,d
		cmp	h
		jnz	dwnlop
escret:	ret
;	
cojt7z:	lhld	hbuf3		
		xchg
		lxi	h,vidram	; VIDO-RAM Start - eine zeile
		lxi	b,lin		; Zeilenlaenge (80 Zeichen)
poslop:	dad	b
		dcr	e
		jnz	poslop	; Schleife bis Zeile erricht ist
		mov	c,d
		dad	b		; Zeilenposition 
		call	crspos_v
		ret
;	
cojt8z:	lxi	d,vidend	; Bildschirm ab Cursor loeschen
		lhld	update
		jmp	filop
;	
cojt9z:	call	clrsub	; Zeile loeschen
		jmp	clrlin_v
;	
cojt10z:	call	clrsub	; Zeile ab Cursor loeschen
		lxi	b,lin
		dad	b
		xchg
		lhld	update
filop:	in	vid_dspen	; Ausgabe mit Strahlrueklauf 
		rrc			; Synchronisieren
		jnc	filop		; auf Blanking warten
		mvi	m,20h		; ASCII Leerzeichen "20"
		inx	h
		mov	a,e
		cmp	l		; Endabfrage
		jnz	filop
		mov	a,d
		cmp	h
		jnz	filop
		ret
;	
clrsub:	lxi	b,lininv	; (BC) negative Zeilenlaenge
		lxi	d,vidend
		lhld	update
		xchg
clrlop:	mov	a,h		; (HL) Zeilenanfang
		cmp	d
		jc	clrend
		jnz	clrcte
		mov	a,l
		cmp	e
		jc	clrend
		jz	clrend
clrcte:	dad	b
		jmp	clrlop
;	
clrend:	ret			; (HL) Anfang Cursorzeile
;
;********************************************************************
;        
.end	


